from genericpath import samefile
import os
import PyQt5
dirname = os.path.dirname(PyQt5.__file__)
plugin_path = os.path.join(dirname, 'plugins', 'platforms')
os.environ['QT_QPA_PLATFORM_PLUGIN_PATH'] = plugin_path
import PyQt5.QtWidgets as Qt# QApplication, QWidget, QMainWindow, QPushButton, QHBoxLayout
import PyQt5.QtGui as QtGui
import PyQt5.QtCore as QtCore
import logging
import sys
import argparse

import abstract_instrument_interface
#import pyThorlabsTC300.driver_virtual
import pyThorlabsTC300.driver

graphics_dir = os.path.join(os.path.dirname(__file__), 'graphics')

##This application follows the model-view-controller paradigm, but with the view and controller defined inside the same object (the GUI)
##The model is defined by the class 'interface', and the view+controller is defined by the class 'gui'. 

class interface(abstract_instrument_interface.abstract_interface):
    """
    Create a high-level interface with the device, validates input data and perform high-level tasks such as periodically reading data from the instrument.
    It uses signals (i.e. QtCore.pyqtSignal objects) to notify whenever relevant data has changes. These signals are typically received by the GUI
    Several general-purpose attributes and methods are defined in the class abstract_interface defined in abstract_instrument_interface
    ...

    Attributes specific for this class (see the abstract class abstract_instrument_interface.abstract_interface for general attributes)
    [LIST TO FINISH]
    ----------
    instrument
        Instance of pyThorlabsTC300.driver.ThorlabsTC300
    connected_device_name : str
        Name of the physical device currently connected to this interface 
    continuous_read : bool 
        When this is set to True, the data from device are acquired continuosly at the rate set by refresh_time       
        settings = {
                'refresh_time': float,      The time interval (in seconds) between consecutive reeading from the device driver (default = 0.2)
                }
    ramp 
        Instance of abstract_instrument_interface.ramp class


    Methods defined in this class (see the abstract class abstract_instrument_interface.abstract_interface for general methods)
    [LIST TO FINISH]
    -------
    refresh_list_devices()
        Get a list of compatible devices from the driver. Store them in self.list_devices, send signal to populate the combobox in the GUI.
    connect_device(device_full_name)
        Connect to the device identified by device_full_name
    disconnect_device()
        Disconnect the currently connected device
    close()
        Closes this interface and calls the close() method of the parent class, which typically calls the disconnect_device method
   
    set_connected_state()
        This method also calls the set_connected_state() method defined in abstract_instrument_interface.abstract_interface
    
        
    update()
        This method also calls the update() method defined in abstract_instrument_interface.abstract_interface

    """

    output = {'Power':0}  #We define this also as class variable. This makes it possible to see which data is produced by this interface without having to create an object

    ## SIGNALS THAT WILL BE USED TO COMMUNICATE WITH THE GUI
    #                                                               | Triggered when ...                                        | Parameter(s) Sent     
    #                                                           #   -----------------------------------------------------------------------------------------------------------------------         
    sig_list_devices_updated = QtCore.pyqtSignal(list)          #   | List of devices is updated                                | List of devices   
    sig_channel_mode_changed = QtCore.pyqtSignal(int,int)       #   | The mode of one channel changed (or was read)             | Channel number, Channel mode [0: Heater; 1: Tec; 2: Constant current; 3: Synchronize with Ch1(only for channel 2)]
    sig_temperature_read = QtCore.pyqtSignal(int,float)         #   | The actual tempereature of a channel was read             | Channel number, Actual Temperature
    sig_current_read = QtCore.pyqtSignal(int,float)             #   | The actual current of a channel was read                  | Channel number, Actual Current
    sig_target_temperature_read = QtCore.pyqtSignal(int,float)  #   | The target temperature of a channel was read              | Channel number, Target Temperature
    sig_target_current_read = QtCore.pyqtSignal(int,float)      #   | The target current of a channel was read                  | Channel number, Target Current
    sig_channel_enabled = QtCore.pyqtSignal(int)                #   | A channel has been enabled                                | Channel number
    sig_enabled_state_change = QtCore.pyqtSignal(int,int)
    sig_change_moving_status = QtCore.pyqtSignal(int)           #   | A step (either of temperature or current) initiated by ramp has started or ended | self.interface.SIG_MOVEMENT_STARTED, self.interface.SIG_MOVEMENT_ENDED
    
    sig_reading = QtCore.pyqtSignal(int)                    #   | Reading status changes                                    | 1 = Started Reading, 2 = Paused Reading, 3 Stopped Reading
    sig_updated_data = QtCore.pyqtSignal(object)            #   | Data is read from instrument                              | Acquired data 
    sig_wavelength = QtCore.pyqtSignal(int)                 #   | Wavelength is changed                                     | Current Wavelength
    sig_min_max_wavelength = QtCore.pyqtSignal(int,int)     #   | Min and max wavelengths supported by this device are read | Current Min and max wavelengths
    sig_refreshtime = QtCore.pyqtSignal(float)              #   | Refresh time is changed                                   | Current Refresh time 
    sig_power_range = QtCore.pyqtSignal(float)              #   | Power range is changed                                    | Current Power range
    sig_auto_power_range = QtCore.pyqtSignal(bool)          #   | Auto power range setting is changed                       | Current Status of auto power range (true/false)
    ##
    # Identifier codes used for view-model communication. Other general-purpose codes are specified in abstract_instrument_interface
    SIG_READING_START = 1
    SIG_READING_PAUSE = 2
    SIG_READING_STOP = 3

    SIG_MOVEMENT_STARTED = 1
    SIG_MOVEMENT_ENDED = 2

    def __init__(self, **kwargs):
        self.output = {'T1':0, 'T2':0, 'i1':0, 'i2':0} 
        ### Default values of settings (might be overwritten by settings saved in .json files later)
        self.settings = {   'refresh_time': 1,
                            'ramp' : {  
                                        'ramp_step_size': 1,            #Increment value of each ramp step
                                        'ramp_wait_1': 1,               #Wait time (in s) after each ramp step
                                        'ramp_send_trigger' : True,     #If true, the function self.func_trigger is called after each 'movement'
                                        'ramp_wait_2': 1,               #Wait time (in s) after each (potential) call to trigger, before doing the new ramp step
                                        'ramp_numb_steps': 10,          #Number of steps in the ramp
                                        'ramp_repeat': 1,               #How many times the ramp is repeated
                                        'ramp_reverse': 1,              #If True (or 1), it repeates the ramp in reverse
                                        'ramp_send_initial_trigger': 1, #If True (or 1), it calls self.func_trigger before starting the ramp
                                        'ramp_reset' : 1                #If True (or 1), it resets the value of the instrument to the initial one after the ramp is done
                                        }
                            }
        self.list_devices = []          #list of devices found 
        self.connected_device_name = ''
        self.continuous_read = False    # When this is set to True, the data from device are acquired continuosly at the rate set by self.refresh_time
        self.channels = [1,2]
        self.mode = [0,0,0] #Stores the mode of each channel [0: Heater; 1: Tec; 2: Constant current; 3: Synchronize with Ch1(only for channel 2)]
                    #The element self.mode[0] is dummy
        self.channel_controlled_by_ramp = 1
        self.modality_names = {0: "Heater", 1: "Tec", 2: "Constant current", 3: "Synchronize with Ch1(only for channel 2)"}
        self.is_enabled = [0,0,0] #Stores whether a channel is enabled (1) or disabled (0)
        self.actual_temperature = [0,0,0] #Stores the latest read temperature of each channel. The element self.actual_temperature[0] is dummy
        self.actual_current = [0,0,0] #Stores the latest read current of each channel. The element self.actual_current[0] is dummy
        self.target_temperature = [0,0,0] #Stores the latest read target temperature of each channel. The element self.target_temperature[0] is dummy
        self.target_current = [0,0,0] #Stores the latest read target current of each channel. The element self.target_current[0] is dummy

        ###
        if ('virtual' in kwargs.keys()) and (kwargs['virtual'] == True):
            pass
            #self.instrument = pyThorlabsPM100x.driver_virtual.ThorlabsPM100x() 
        else:    
            self.instrument = pyThorlabsTC300.driver.ThorlabsTC300() 
        ###
        super().__init__(**kwargs)

        # Setting up the ramp object, which is defined in the package abstract_instrument_interface
        self.ramp = abstract_instrument_interface.ramp(interface=self)  
        self.ramp.set_ramp_settings(self.settings['ramp'])
        self.ramp.set_ramp_functions(func_move = self.increase_target_by,
                                     func_check_step_has_ended = self.step_has_ended, 
                                     func_trigger = super().update(), 
                                     func_trigger_continue_ramp = None,
                                     func_set_value = self.set_target_value, 
                                     func_read_current_value = self.get_target_value, 
                                     list_functions_step_not_ended = [],  
                                     list_functions_step_has_ended = [self.read_output_channel],#lambda:self.end_movement(send_signal=False)],  
                                     list_functions_ramp_started = [self.pause_reading],
                                     list_functions_ramp_ended = [self.start_reading])
        self.ramp.sig_ramp.connect(self.on_ramp_state_changed)

        self.refresh_list_devices()   
        
    def refresh_list_devices(self):
        '''
        Get a list of all devices connected, by using the method list_devices() of the driver. For each device obtain its identity and its address.
        '''     
        self.logger.info(f"Looking for devices...") 
        list_valid_devices = self.instrument.list_devices() #Then we read the list of devices
        self.logger.info(f"Found {len(list_valid_devices)} devices.") 
        self.list_devices = list_valid_devices
        self.send_list_devices()

    def send_list_devices(self):
        if(len(self.list_devices)>0):
            list_IDNs_and_devices = [dev[0] + " --> " + dev[1] for dev in self.list_devices] 
        else:
            list_IDNs_and_devices = []
        self.sig_list_devices_updated.emit(list_IDNs_and_devices)

    def connect_device(self,device_full_name):
        if(device_full_name==''): 
            self.logger.error("No valid device has been selected.")
            return
        self.set_connecting_state()
        device_addr = device_full_name.split(' --> ')[0].lstrip()   # We extract the device address from the device name
        self.logger.info(f"Connecting to device {device_addr}...")
        try:
            (ID, Msg) = self.instrument.connect_device(device_addr)      # Try to connect by using the method connect_device of the device driver
            if(ID==1):  #If connection was successful
                self.logger.info(f"Connected to device {device_addr}.")
                self.connected_device_name = device_addr
                self.set_connected_state()
            else: #If connection was not successful
                self.logger.error(f"Error: {Msg}")
                self.set_disconnected_state()
        except Exception as e:
            self.logger.error(f"Error: {e}")
            self.set_disconnected_state()

    def disconnect_device(self):
        self.logger.info(f"Disconnecting from device {self.connected_device_name}...")
        (ID, Msg) = self.instrument.disconnect_device()
        if(ID==1): # If disconnection was successful
            self.logger.info(f"Disconnected from device {self.connected_device_name}.")
            self.continuous_read = 0 # We set this variable to 0 so that the continuous reading from the powermeter will stop
            self.set_disconnected_state()
        else: #If disconnection was not successful
            self.logger.error(f"Error: {Msg}")
            self.set_disconnected_state() #When disconnection is not succeful, it is typically because the device alredy lost connection
                                          #for some reason. In this case, it is still useful to have all widgets reset to disconnected state      
    def close(self,**kwargs):
        super().close(**kwargs) 
        self.settings['ramp'] = self.ramp.settings

    def on_ramp_state_changed(self,status):
        '''
        Slot for signals coming from the ramp object
        '''
        if status == self.ramp.SIG_RAMP_STARTED:
            #self.set_moving_state()
            self.settings['ramp'] = self.ramp.settings
        if status == self.ramp.SIG_RAMP_ENDED:
            pass
            #self.set_non_moving_state()
        
    def toggle_enable_disable(self,channel):
        if self.is_enabled[channel]:
            self.disable_channel(channel)
        else:
            self.enable_channel(channel)

    def set_connected_state(self):
        super().set_connected_state()
        self.read_status_channels()
        
        #self.read_min_max_wavelength()
        #self.read_wavelength()
        #self.read_status_power_autorange()
        #self.set_auto_power_range(self.settings['auto_power_range'])
        #self.read_power_range()
        self.start_reading()

    def read_status_channels(self,channels=None):
        #This is typically called after connection, to check different properties of the channels and fire the corresponding signals
        # if channels is not specified, it does everything for all channels in channels, otherwise it only does it for the channel specified by channels, which must be a valid integer or a list of integers
        if channels == None : channels = self.channels
        if not(isinstance(channels, list)): channels = [channels]
        self.check_if_channel_is_enabled(channels)
        self.read_mode_channel(channels)
        self.read_output_channel(channels)
        self.read_target_channel(channels)

        return

    def check_if_channel_is_enabled(self,channels=None):
        # if channels is not specified, it does everything for all channels in channels, otherwise it only does it for the channel specified by channels, which must be a valid integer or a list of integers
        if channels == None : channels = self.channels
        if not(isinstance(channels, list)): channels = [channels]
        for channel in channels:
            self.is_enabled[channel] = self.is_enabled[channel] #This is just a placeholder
            self.sig_enabled_state_change.emit(channel,self.is_enabled[channel])
        return

    def read_mode_channel(self,channels=None):
        # if channels is not specified, it does everything for all channels in channels, otherwise it only does it for the channel specified by channels, which must be a valid integer or a list of integers
        #[0: Heater; 1: Tec; 2: Constant current; 3: Synchronize with Ch1(only for channel 2)]
        if channels == None : channels = self.channels
        if not(isinstance(channels, list)): channels = [channels]
        
        for channel in channels:
            self.mode[channel] = self.instrument.get_mode_channel(channel)
            self.sig_channel_mode_changed.emit(channel,self.mode[channel])
            self.logger.info(f"Channel {channel} is in modality: {self.modality_names[self.mode[channel]]}.")

    def set_mode_channel(self,channel,mode):
        #[0: Heater; 1: Tec; 2: Constant current; 3: Synchronize with Ch1(only for channel 2)]
        try:
            self.instrument.set_mode_channel(channel,mode)
        except Exception as e:
            self.logger.error(f"An error occurred while setting the mode of channel {channel}: {e}")
        self.read_mode_channel(channel)
        self.read_target_channel() #This will update the target quantity displayed in the GUI, which depends on the channel modality

    def read_output_channel(self,channels=None):
        # if channels is not specified, it does everything for all channels in channels, otherwise it only does it for the channel specified by channels, which must be a valid integer or a list of integers
        
        if channels == None : channels = self.channels
        if not(isinstance(channels, list)): channels = [channels]
        for channel in channels:
            if self.mode[channel] == 0:
                T = self.instrument.get_actual_temperature(channel)
                self.actual_temperature[channel] = T
                self.sig_temperature_read.emit(channel,T)
                self.output['T'+str(channel)] = T
            elif self.mode[channel] == 2:
                I = self.instrument.get_actual_current(channel)
                self.actual_current[channel] = I
                self.sig_current_read.emit(channel,I)
                self.output['T'+str(channel)] = I

    def read_target_channel(self,channels=None):
        # if channels is not specified, it does everything for all channels in channels, otherwise it only does it for the channel specified by channels, which must be a valid integer or a list of integers
        
        if channels == None : channels = self.channels
        if not(isinstance(channels, list)): channels = [channels]
        for channel in channels:
            if self.mode[channel] == 0:
                T = self.instrument.get_target_temperature(channel)
                self.target_temperature[channel] = T
                self.sig_target_temperature_read.emit(channel,T)
            elif self.mode[channel] == 2:
                I = self.instrument.get_target_current(channel)
                self.target_current[channel] = I
                self.sig_target_current_read.emit(channel,I)

    def set_target_channel(self,channel,target):
        try: 
            target = float(target)
        except ValueError:
            self.logger.error(f"The target time must be a valid number.")
            self.read_target_channel(channel)
            return False
        
        if self.mode[channel] == 0:
            self.logger.info(f"Setting the target of channel {channel} to {target} C")
            try:
                self.instrument.set_target_temperature(channel,target)
            except Exception as e:
                self.logger.error(f"An error occurred while setting target temperature for channel {channel}: {e}")
        elif self.mode[channel] == 2:
            self.logger.info(f"Setting the target of channel {channel} to {target} mA")
            try:
                self.instrument.set_target_current(channel,target)
            except Exception as e:
                self.logger.error(f"An error occurred while setting target current for channel {channel}: {e}")
        self.read_target_channel(channel)

    def enable_channel(self,channel):
        self.logger.info(f"Enabling channel {channel}...")
        try:
            self.instrument.enable_disable_channel(channel,1)
        except Exception as e:
            self.logger.error(f"An error occurred while enabling channel {channel}: {e}")
        self.logger.info(f"Channel {channel} enabled")
        self.is_enabled[channel] = 1
        self.sig_enabled_state_change.emit(channel,1)

    def disable_channel(self,channel):
        self.logger.info(f"Disabling channel {channel}...")
        try:
            self.instrument.enable_disable_channel(channel,0)
        except Exception as e:
            self.logger.error(f"An error occurred while disabling channel {channel}: {e}")
        self.logger.info(f"Channel {channel} disabled")
        self.is_enabled[channel] = 0
        self.sig_enabled_state_change.emit(channel,0)

    def set_refresh_time(self, refresh_time):
        try: 
            refresh_time = float(refresh_time)
            if self.settings['refresh_time'] == refresh_time: #in this case the number in the refresh time edit box is the same as the refresh time currently stored
                return True
        except ValueError:
            self.logger.error(f"The refresh time must be a valid number.")
            self.sig_refreshtime.emit(self.settings['refresh_time'])
            return False
        if refresh_time < 0.001:
            self.logger.error(f"The refresh time must be positive and >= 1ms.")
            self.sig_refreshtime.emit(self.settings['refresh_time'])
            return False
        self.logger.info(f"The refresh time is now {refresh_time} s.")
        self.settings['refresh_time'] = refresh_time
        self.sig_refreshtime.emit(self.settings['refresh_time'])
        return True

    ### Functions mainly used for interface with ramp

    def increase_target_by(self,step,channel = None):
        if channel == None: channel = self.channel_controlled_by_ramp
        if self.mode[channel] == 0:
            current_target = self.target_temperature[channel]
        elif self.mode[channel] == 2:
            current_target = self.target_current[channel]
             
        next_target = current_target + step
        self.set_target_channel(channel,next_target)

    def get_target_value(self,channel = None):
        if channel == None: channel = self.channel_controlled_by_ramp
        self.read_target_channel(channel)
        current_target = self.target_current[channel]
        return current_target

    def set_target_value(self,target,channel = None):
        if channel == None: channel = self.channel_controlled_by_ramp
        self.set_target_channel(channel,target)

    def step_has_ended(self):
        # This function is used by the ramp to check if a step has ended. For now it is a placeholder and always returns True
        return True


    ###

    #def set_wavelength(self, wl):
    #    try:
    #        if int(self.instrument.wavelength) == int(float(wl)): #in this case the number in the refresh time edit box is the same as the wavelength currently set
    #                return True
    #        self.logger.info(f"Setting the wavelength to {wl} for the device {self.connected_device_name}...")
    #    except ValueError as e:
    #        self.logger.error(f"The wavelength must be a valid number.")
    #        self.sig_wavelength.emit(self.instrument.wavelength)
    #        return False
    #    try: 
    #        self.instrument.wavelength = int(float(wl))
    #        self.logger.info(f"Wavelength set correctly.")
    #    except Exception as e:
    #        self.logger.error(f"An error occurred while setting the wavelength: {e}")
    #        self.sig_wavelength.emit(self.instrument.wavelength)
    #        return False
    #    self.read_power_range() #The boundaries of the power ranges might change when the wavelength is changed, so we need to update it after changing the wavelength
    #    return True
  
    #def read_wavelength(self):
    #    self.logger.info(f"Reading current wavelength from device {self.connected_device_name}...") 
    #    self.wavelength = int(self.instrument.wavelength)
    #    if self.wavelength == None:
    #        self.logger.error(f"An error occurred while reading the wavelength from this device.")
    #        return
    #    self.sig_wavelength.emit(self.instrument.wavelength)
    #    self.logger.info(f"Current wavelength is {self.wavelength}.") 
    #    return
      
    #def read_min_max_wavelength(self):
    #    self.logger.info(f"Reading min and max wavelength supported by device {self.connected_device_name}...") 
    #    self.min_max_wls =  (self.instrument.min_wavelength,self.instrument.max_wavelength)
    #    self.sig_min_max_wavelength.emit(self.min_max_wls[0],self.min_max_wls[1])
    #    self.logger.info(f"Wavelength range: {self.min_max_wls[0]}-{self.min_max_wls[1]} nm") 

    #def change_power_range(self,direction):
    #    if direction == +1:
    #        string = 'increase'
    #    else: 
    #        string = 'decrease'
    #    self.logger.info(f"Trying to {string} the power range...")
    #    self.instrument.move_to_next_power_range(direction)
    #    self.read_power_range()
    #    return

    #def read_power_range(self):
    #    self.logger.info(f"Reading current power range from device {self.connected_device_name}...") 
    #    self.power_range = self.instrument.power_range
    #    if self.power_range == None:
    #        self.logger.error(f"An error occurred while reading the power range from this device.")
    #        return
    #    self.sig_power_range.emit(self.power_range)
    #    self.logger.info(f"Current power range is {self.power_range}.") 

    #def set_auto_power_range(self,auto_power_range):
    #    auto_power_range = bool(auto_power_range)
    #    status_string = 'ON' if auto_power_range else 'OFF'
    #    self.logger.info(f"Setting the auto-ranging function to {status_string} for the device {self.connected_device_name}...")    
    #    try:
    #        self.instrument.auto_power_range  = auto_power_range
    #        self.sig_auto_power_range.emit(auto_power_range)
    #        self.logger.info(f"Setting changed succesfully.")
    #        self.settings['auto_power_range'] = auto_power_range
    #    except Exception as e:
    #        self.logger.error(f"An error occurred while setting the auto-ranging status: {e}")
    #    #self.read_auto_power_range()
    #    self.read_power_range()

    #def read_auto_power_range(self):
    #    self.logger.info(f"Reading the status of the auto-ranging function for the device {self.connected_device_name}...")   
    #    try:
    #        auto_power_range = self.instrument.auto_power_range
    #        status_string = 'ON' if auto_power_range else 'OFF'
    #        self.sig_auto_power_range.emit(auto_power_range)
    #        self.logger.info(f"The auto-ranging function is currently set to {status_string}.")
    #        self.settings['auto_power_range'] = auto_power_range
    #        return auto_power_range
    #    except Exception as e:
    #        self.logger.error(f"An error occurred while reading the auto-ranging status: {e}")

    #def set_zero_powermeter(self):
    #    try: 
    #        ID = self.instrument.set_zero()
    #        self.logger.info(f"Zero-ing the device {self.connected_device_name}...")
    #        if ID == 1:
    #            self.logger.info(f"Device was succesfully zeroed.")
    #        else:
    #            self.logger.error(f"An error occurred while zero-ing this device.")
    #    except Exception as e:
    #        self.logger.error(f"An error occurred while zero-ing this device: {e}")
    #    return

    def start_reading(self):
        if(self.instrument.connected == False):
            self.logger.error(f"No device is connected.")
            return
        #self.logger.info(f"Updating wavelength and refresh time before starting reading...")       
        self.sig_reading.emit(self.SIG_READING_START) # This signal will be caught by the GUI
        self.continuous_read = True #Until this variable is set to True, the function UpdatePower will be repeated continuosly 
        self.logger.info(f"Starting reading from device {self.connected_device_name}...")
        # Call the function self.update(), which will do stome suff (read power and store it in a global variable) and then call itself continuosly until the variable self.continuous_read is set to False
        self.update()
        return
 
    def pause_reading(self):
        #Sets self.continuous_read to False (this will force the function update() to stop calling itself)
        self.continuous_read = False
        self.logger.info(f"Paused reading from device {self.connected_device_name}.")
        self.sig_reading.emit(self.SIG_READING_PAUSE) # This signal will be caught by the GUI
        return

    def stop_reading(self):
        #Sets self.continuous_read to False (this will force the function update() to stop calling itself) and delete all accumulated data
        self.continuous_read = False
        self.stored_data = []
        self.logger.info(f"Stopped reading from device {self.connected_device_name}. All stored data have been deleted.")
        self.sig_reading.emit(self.SIG_READING_PAUSE) # This signal will be caught by the GUI
        # ...
        return
        
    def update(self):
        '''

        '''
        if(self.continuous_read == True):
            self.read_output_channel()
            #(currentPower,power_units) = self.instrument.power
            #self.output['Power'] = currentPower
            #self.power_units = power_units
            #self.stored_data.append(currentPower)
            #self.output['PowerUnits'] = power_units

            super().update()    

            #self.sig_updated_data.emit([currentPower, power_units])
            QtCore.QTimer.singleShot(int(self.settings['refresh_time']*1e3), self.update)
           
        return
    
    
class gui(abstract_instrument_interface.abstract_gui):
     
    def __init__(self,interface,parent):
        """
        Attributes specific for this class (see the abstract class abstract_instrument_interface.abstract_gui for general attributes)
        ----------

        """
        super().__init__(interface,parent)

        self.initialize()

    def initialize(self):
        self.create_widgets()
        self.connect_widgets_events_to_functions()

        ### Call the initialize method of the super class. 
        super().initialize()

        ### Connect signals from model to event slots of this GUI
        self.interface.sig_list_devices_updated.connect(self.on_list_devices_updated)
        self.interface.sig_connected.connect(self.on_connection_status_change) 
        self.interface.sig_channel_mode_changed.connect(self.on_channel_mode_changed)
        self.interface.sig_temperature_read.connect(self.on_actual_temperature_changed)
        self.interface.sig_current_read.connect(self.on_actual_current_changed)
        self.interface.sig_target_temperature_read.connect(self.on_target_temperature_changed)
        self.interface.sig_target_current_read.connect(self.on_target_current_changed)
        self.interface.sig_enabled_state_change.connect(self.on_enabled_state_change)
        #self.interface.sig_reading.connect(self.on_reading_status_change) 
        #self.interface.sig_updated_data.connect(self.on_data_change) 
        #self.interface.sig_refreshtime.connect(self.on_refreshtime_change)
        #self.interface.sig_wavelength.connect(self.on_wavelength_change)
        #self.interface.sig_min_max_wavelength.connect(self.on_min_max_wavelength_update)
        #self.interface.sig_auto_power_range.connect(self.on_auto_power_range_change)
        #self.interface.sig_power_range.connect(self.on_power_range_change)
        #self.interface.sig_close.connect(self.on_close)

        ### SET INITIAL STATE OF WIDGETS
        #self.edit_RefreshTime.setText(f"{self.interface.settings['refresh_time']:.3f}")
        self.interface.send_list_devices()  
        #self.on_connection_status_change(self.interface.SIG_DISCONNECTED) #When GUI is created, all widgets are set to the "Disconnected" state              
        ###

    def create_panel_channel(self,channel_name = 'CH'):

        QGroupBoxStyle = """ {
            font: bold;
            border: 1px solid silver;
            border-radius: 6px;
            margin-top: 6px;
        }"""

        QGroupBoxTitleStyle = """::title {
                                subcontrol-origin: margin;
                                left: 17px;
                                padding: -5px 5px 0px 5px;
                            }"""

        groupbox_main = Qt.QGroupBox(channel_name)
        groupbox_main.setObjectName(channel_name)

        string = "QGroupBox"
        groupbox_main.setStyleSheet(string + QGroupBoxStyle + string + QGroupBoxTitleStyle) #This line changes the style of ONLY this QWdiget

        vbox_main = Qt.QVBoxLayout()

        hbox_row1 = Qt.QHBoxLayout()
        button_onoff = Qt.QPushButton("OFF")
        button_onoff.setCheckable(True)
        button_onoff.setStyleSheet("background-color: red; font: bold;")
        label_mode = Qt.QLabel('Mode:')
        combo_mode = Qt.QComboBox()
        combo_mode.addItems([self.interface.modality_names[0],self.interface.modality_names[2]]) 
        widgets_row1 = {'button_onoff':button_onoff,'label_mode':label_mode,'combo_mode':combo_mode}
        widgets_row1_stretches = [0]*len(widgets_row1)
        for w,s in zip(widgets_row1.values(),widgets_row1_stretches):
            hbox_row1.addWidget(w,stretch=s)
        hbox_row1.addStretch(1)

        hbox_row2 = Qt.QHBoxLayout()
        label_value = Qt.QLabel('0')
        label_value.setAlignment(QtCore.Qt.AlignCenter)
        label_value.setStyleSheet("font-weight: bold")
        label_value.setFont(QtGui.QFont('Arial', 10))
        widgets_row2 = {'label_value':label_value}
        widgets_row2_stretches = [1]*len(widgets_row2)
        for w,s in zip(widgets_row2.values(),widgets_row2_stretches):
            hbox_row2.addWidget(w,stretch=s)

        hbox_row3 = Qt.QHBoxLayout()
        label_target = Qt.QLabel('Target')
        edit_target = Qt.QLineEdit(' ')
        label_target_units = Qt.QLabel('')
        widgets_row3 = {'label_target':label_target,'edit_target':edit_target,'label_target_units':label_target_units}
        widgets_row3_stretches = [0]*len(widgets_row3)
        for w,s in zip(widgets_row3.values(),widgets_row3_stretches):
            hbox_row3.addWidget(w,stretch=s)

        box_stretches = [0,1,0]
        boxes = [hbox_row1,hbox_row2,hbox_row3]
        for box,s in zip(boxes,box_stretches):
            vbox_main.addLayout(box,stretch=s)  
        #vbox_main.addStretch(1)

        groupbox_main.setLayout(vbox_main)
        widgets = {**widgets_row1, **widgets_row2, **widgets_row3}

        return groupbox_main,widgets

    def create_widgets(self):
        """
        Creates all widgets and layout for the GUI. Any Widget and Layout must assigned to self.containter, which is a pyqt Layout object
        """ 
        self.container = Qt.QVBoxLayout()

        self.widgets_channels =[0,0,0]

        hbox1, widgets_dict = self.create_panel_connection_listdevices()
        for key, val in widgets_dict.items(): 
            setattr(self,key,val) 

        hbox2 = Qt.QHBoxLayout()

        box_CH1,widgets_CH1 = self.create_panel_channel(channel_name = 'CH1')
        box_CH2,widgets_CH2 = self.create_panel_channel(channel_name = 'CH2')

        self.widgets_channels[1] = widgets_CH1 
        self.widgets_channels[2] = widgets_CH2 
  
        for box in [box_CH1,box_CH2]:
            hbox2.addWidget(box,stretch=1) 
        #hbox2.addStretch(1)

        self.ramp_groupbox = abstract_instrument_interface.ramp_gui(ramp_object=self.interface.ramp)     

        for box in [hbox1,hbox2]:
            self.container.addLayout(box) 
        self.container.addWidget(self.ramp_groupbox)
        #self.container.addStretch(1)

        ## Widgets for which we want to constraint the width by using sizeHint()
        #widget_list = [self.button_StopReading,self.label_RefreshTime,self.label_Power,self.button_SetZeroPowermeter,self.label_WavelengthUnits,self.label_PowerRange,self.box_PowerRangeAuto]
        #for w in widget_list:
        #    w.setMaximumSize(w.sizeHint())
        self.widgets_enabled_when_connected =[]
        #self.widgets_enabled_when_connected = [self.button_SetZeroPowermeter,self.edit_Wavelength,self.edit_PowerRange,self.box_PowerRangeAuto, 
        #                                       self.button_IncreasePowerRange,self.button_DecreasePowerRange,self.button_StartPauseReading,self.button_StopReading]
        self.widgets_enabled_when_disconnected = []
        #self.widgets_enabled_when_disconnected = [self.combo_Devices,self.button_RefreshDeviceList]

    def connect_widgets_events_to_functions(self):
        self.button_RefreshDeviceList.clicked.connect(self.click_button_refresh_list_devices)
        self.button_ConnectDevice.clicked.connect(self.click_button_connect_disconnect)
        for channel in [1,2]:
            self.widgets_channels[channel]['button_onoff'].clicked.connect(lambda x,y=channel:self.click_button_on_off(y))
            self.widgets_channels[channel]['combo_mode'].activated.connect(lambda x,y=channel:self.click_combo_mode(y))
            self.widgets_channels[channel]['edit_target'].returnPressed.connect(lambda y=channel: self.press_enter_edit_target(y))
        #self.edit_RefreshTime.returnPressed.connect(self.press_enter_refresh_time)


###########################################################################################################
### Event Slots. They are normally triggered by signals from the model, and change the GUI accordingly  ###
###########################################################################################################

    def on_connection_status_change(self,status):
        if status == self.interface.SIG_DISCONNECTED:
            self.disable_widget(self.widgets_enabled_when_connected)
            self.enable_widget(self.widgets_enabled_when_disconnected)
            #self.edit_PowerRange.setText('')
            #self.edit_Wavelength.setText('')
            #self.label_Wavelength.setText(f"Wavelength: ")
            self.button_ConnectDevice.setText("Connect")
            #self.edit_Power.setText('')
        if status == self.interface.SIG_DISCONNECTING:
            self.disable_widget(self.widgets_enabled_when_connected)
            self.enable_widget(self.widgets_enabled_when_disconnected)
            self.button_ConnectDevice.setText("Disconnecting...")
        if status == self.interface.SIG_CONNECTING:
            self.disable_widget(self.widgets_enabled_when_connected)
            self.enable_widget(self.widgets_enabled_when_disconnected)
            self.button_ConnectDevice.setText("Connecting...")
        if status == self.interface.SIG_CONNECTED:
            self.enable_widget(self.widgets_enabled_when_connected)
            self.disable_widget(self.widgets_enabled_when_disconnected)
            self.button_ConnectDevice.setText("Disconnect")

    def on_enabled_state_change(self,channel,status):
        if status == 1:
            if not(self.widgets_channels[channel]['button_onoff'].isChecked()):
                self.widgets_channels[channel]['button_onoff'].toggle()
            self.widgets_channels[channel]['button_onoff'].setText('ON')
            self.widgets_channels[channel]['button_onoff'].setStyleSheet("background-color: green; font: bold;")
        if status == 0:
            if self.widgets_channels[channel]['button_onoff'].isChecked():
                self.widgets_channels[channel]['button_onoff'].toggle()
            self.widgets_channels[channel]['button_onoff'].setText('OFF')
            self.widgets_channels[channel]['button_onoff'].setStyleSheet("background-color: red; font: bold;")

    def on_channel_mode_changed(self,channel,mode):
        #{0: "Heater", 1: "Tec", 2: "Constant current", 3: "Synchronize with Ch1(only for channel 2)"}
        if mode in [0,2]:
            self.widgets_channels[channel]['combo_mode'].setCurrentText(self.interface.modality_names[mode])

    def on_actual_temperature_changed(self,channel,T):
        if self.interface.mode[channel] == 0:
            self.widgets_channels[channel]['label_value'].setText(f'{T:.2f} C')
        return

    def on_actual_current_changed(self,channel,I):
        if self.interface.mode[channel] == 2:
            self.widgets_channels[channel]['label_value'].setText(f'{I:.2f} mA')
        return

    def on_target_temperature_changed(self,channel,T):
        if self.interface.mode[channel] == 0:
            self.widgets_channels[channel]['edit_target'].setText(f'{T:.2f}')
            self.widgets_channels[channel]['label_target_units'].setText(f'C')
        return

    def on_target_current_changed(self,channel,I):
        if self.interface.mode[channel] == 2:
            self.widgets_channels[channel]['edit_target'].setText(f'{I:.2f}')
            self.widgets_channels[channel]['label_target_units'].setText(f'mA')
        return

    def on_reading_status_change(self,status):
        if status == self.interface.SIG_READING_PAUSE:
            self.button_StartPauseReading.setIcon(QtGui.QIcon(os.path.join(graphics_dir,'play.png')))
        if status == self.interface.SIG_READING_START:
            self.button_StartPauseReading.setIcon(QtGui.QIcon(os.path.join(graphics_dir,'pause.png')))
        if status == self.interface.SIG_READING_STOP: 
            self.button_StartPauseReading.setIcon(QtGui.QIcon(os.path.join(graphics_dir,'play.png')))

    def on_list_devices_updated(self,list_devices):
        self.combo_Devices.clear()  #First we empty the combobox  
        self.combo_Devices.addItems(list_devices) 

    #def on_data_change(self,data):
    #    #Data is (in this case) a string
    #    current_power_string = f"{data[0]:.2e}" + ' ' +  data[1]
    #    self.edit_Power.setText(current_power_string)

    #def on_refreshtime_change(self,value):
    #    self.edit_RefreshTime.setText(f"{value:.3f}")

    #def on_wavelength_change(self,value):
    #    self.edit_Wavelength.setText(str(int(value)))

    #def on_min_max_wavelength_update(self,min,max):
    #    self.label_Wavelength.setText(f"Wavelength<br>(<b>{min}-{max}</b>): ")
    
    #def on_power_range_change(self,value):
    #    self.edit_PowerRange.setText(f"{value:.2e}")
    #    self.edit_PowerRange.setCursorPosition(1)

    #def on_auto_power_range_change(self,value):
    #    self.set_auto_power_range_state(value)
    #    self.box_PowerRangeAuto.setChecked(value)

    #def on_close(self):
    #    return

    #def set_auto_power_range_state(self,auto_power_range):
    #    if auto_power_range:
    #        self.disable_widget([self.edit_PowerRange,self.button_IncreasePowerRange, self.button_DecreasePowerRange])
    #    else:
    #        self.enable_widget([self.edit_PowerRange,self.button_IncreasePowerRange, self.button_DecreasePowerRange])

#######################
### END Event Slots ###
#######################

            
###################################################################################################################################################
### GUI Events Functions. They are triggered by direct interaction with the GUI, and they call methods of the interface (i.e. the model) object.###
###################################################################################################################################################

    def click_button_refresh_list_devices(self):
        self.interface.refresh_list_devices()

    def click_button_connect_disconnect(self):
        if(self.interface.instrument.connected == False): # We attempt connection   
            device_full_name = self.combo_Devices.currentText() # Get the device name from the combobox
            self.interface.connect_device(device_full_name)
        elif(self.interface.instrument.connected == True): # We attempt disconnection
            self.interface.disconnect_device()

    def click_button_on_off(self,channel):
        self.interface.toggle_enable_disable(channel)

    def click_combo_mode(self,channel):
        text = self.widgets_channels[channel]['combo_mode'].currentText()
        keys = [k for k, v in self.interface.modality_names.items() if v == text]
        mode = keys[0]
        self.interface.set_mode_channel(channel,mode)

    def press_enter_edit_target(self,channel):
        target = self.widgets_channels[channel]['edit_target'].text()
        self.interface.set_target_channel(channel,target)

    #def click_box_PowerRangeAuto(self, state):
    #    if state == QtCore.Qt.Checked:
    #        status_bool = True
    #    else:
    #        status_bool = False
    #    self.interface.set_auto_power_range(status_bool)

    #def click_button_set_zero_powermeter(self):
    #    self.interface.set_zero_powermeter()

    #def press_enter_wavelength(self):
    #    return self.interface.set_wavelength(self.edit_Wavelength.text())
        
    #def click_button_change_power_range(self,direction):
    #    self.interface.change_power_range(direction)
       
    #def click_button_StartPauseReading(self): 
    #    if(self.interface.continuous_read == False):
    #        if not(self.press_enter_refresh_time()): #read the current value in the refresh_time textbox, and validates it. The function returns True/False if refresh_time was valid
    #            return
    #        if not(self.press_enter_wavelength()): #read the current value in the wavelength textbox, and validates it. The function returns True/False if refresh_time was valid
    #            return
    #        self.interface.start_reading()
    #    elif (self.interface.continuous_read == True):
    #        self.interface.pause_reading()
    #    return

    #def click_button_StopReading(self):
    #    self.interface.stop_reading()

    #def press_enter_refresh_time(self):
    #    return self.interface.set_refresh_time(self.edit_RefreshTime.text())


#################################
### END GUI Events Functions ####
#################################


            
#################################################################################################

class MainWindow(Qt.QWidget):
    def __init__(self):
        super().__init__()
        self.setWindowTitle(__package__)

    def closeEvent(self, event):
        #if self.child:
        pass#self.child.close()

#################################################################################################

def main():
    parser = argparse.ArgumentParser(description = "",epilog = "")
    parser.add_argument("-s", "--decrease_verbose", help="Decrease verbosity.", action="store_true")
    parser.add_argument('-virtual', help=f"Initialize the virtual driver", action="store_true")
    args = parser.parse_args()
    virtual = args.virtual
    
    app = Qt.QApplication(sys.argv)
    window = MainWindow()
    Interface = interface(app=app,virtual=virtual) 
    Interface.verbose = not(args.decrease_verbose)
    app.aboutToQuit.connect(Interface.close) 
    view = gui(interface = Interface, parent=window) #In this case window is the parent of the gui
    window.show()
    app.exec()# Start the event loop.

if __name__ == '__main__':
    main()

#################################################################################################




from ctypes import *
import ctypes.util as util
import os

libname = "TC300COMMANDLIB_win64"
filename = util.find_library(libname )

#region import dll functions

if (filename is not None):
        lib = windll.LoadLibrary(filename)
else:
    filename = f"%s/{libname }.dll" % os.path.dirname(__file__)
    lib = windll.LoadLibrary(filename)
    if (lib is None):
        filename = f"%s/{libname}" % os.path.dirname(sys.argv[0])
        lib = windll.LoadLibrary(filename)
        if (lib is None):
            raise Exception("Could not find valid .dll file")

TC300Lib = lib
#TC300Lib = cdll.LoadLibrary("./TC300COMMANDLIB_win64.dll")

"""common command
"""
List = TC300Lib.List
List.restype = c_int
List.argtypes = [c_char_p, c_int]

Open = TC300Lib.Open
Open.restype = c_int
Open.argtypes = [c_char_p,c_int,c_int]

IsOpen = TC300Lib.IsOpen
IsOpen.restype = c_int
IsOpen.argtypes = [c_char_p]

Close = TC300Lib.Close
Close.restype = c_int
Close.argtypes = [c_int]

GetHandle = TC300Lib.GetHandle
GetHandle.restype = c_int
GetHandle.argtypes = [c_char_p]

"""device command
"""
GetId = TC300Lib.GetId
GetId.restype = c_int
GetId.argtypes = [c_int,c_char_p]

GetStatus = TC300Lib.GetStatus
GetStatus.restype = c_int
GetStatus.argtypes = [c_int,POINTER(c_ubyte)]

EnableChannel = TC300Lib.EnableChannel
EnableChannel.restype = c_int
EnableChannel.argtypes = [c_int,c_int,c_ubyte]

GetChannels = TC300Lib.GetChannels
GetChannels.restype = c_int
GetChannels.argtypes = [c_int,POINTER(c_int)]

CopyParameters = TC300Lib.CopyParameters
CopyParameters.restype = c_int
CopyParameters.argtypes = [c_int]

SetChannels = TC300Lib.SetChannels
SetChannels.restype = c_int
SetChannels.argtypes = [c_int,c_int]

GetT0Constant = TC300Lib.GetT0Constant
GetT0Constant.restype = c_int
GetT0Constant.argtypes = [c_int,c_int,POINTER(c_float)]

SetT0Constant = TC300Lib.SetT0Constant
SetT0Constant.restype = c_int
SetT0Constant.argtypes = [c_int,c_int,c_float]

GetR0Constant = TC300Lib.GetR0Constant
GetR0Constant.restype = c_int
GetR0Constant.argtypes = [c_int,c_int,POINTER(c_float)]

SetR0Constant = TC300Lib.SetR0Constant
SetR0Constant.restype = c_int
SetR0Constant.argtypes = [c_int,c_int,c_float]

GetHartAConstant = TC300Lib.GetHartAConstant
GetHartAConstant.restype = c_int
GetHartAConstant.argtypes = [c_int,c_int,POINTER(c_float)]

SetHartAConstant = TC300Lib.SetHartAConstant
SetHartAConstant.restype = c_int
SetHartAConstant.argtypes = [c_int,c_int,c_float]

GetHartBConstant = TC300Lib.GetHartBConstant
GetHartBConstant.restype = c_int
GetHartBConstant.argtypes = [c_int,c_int,POINTER(c_float)]

SetHartBConstant = TC300Lib.SetHartBConstant
SetHartBConstant.restype = c_int
SetHartBConstant.argtypes = [c_int,c_int,c_float]

GetHartCConstant = TC300Lib.GetHartCConstant
GetHartCConstant.restype = c_int
GetHartCConstant.argtypes = [c_int,c_int,POINTER(c_float)]

SetHartCConstant = TC300Lib.SetHartCConstant
SetHartCConstant.restype = c_int
SetHartCConstant.argtypes = [c_int,c_int,c_float]

GetDarkStatus = TC300Lib.GetDarkStatus
GetDarkStatus.restype = c_int
GetDarkStatus.argtypes = [c_int,POINTER(c_int)]

SetDarkStatus = TC300Lib.SetDarkStatus
SetDarkStatus.restype = c_int
SetDarkStatus.argtypes = [c_int,c_int]

GetQuietMode = TC300Lib.GetQuietMode
GetQuietMode.restype = c_int
GetQuietMode.argtypes = [c_int,POINTER(c_int)]

SetQuietMode = TC300Lib.SetQuietMode
SetQuietMode.restype = c_int
SetQuietMode.argtypes = [c_int,c_int]

GetActualTemperature = TC300Lib.GetActualTemperature
GetActualTemperature.restype = c_int
GetActualTemperature.argtypes = [c_int,c_int,POINTER(c_float)]

SetTargetTemperature = TC300Lib.SetTargetTemperature
SetTargetTemperature.restype = c_int
SetTargetTemperature.argtypes = [c_int,c_int,c_float]

GetTargetTemperature = TC300Lib.GetTargetTemperature
GetTargetTemperature.restype = c_int
GetTargetTemperature.argtypes = [c_int,c_int,POINTER(c_float)]

SetMaxTemperature = TC300Lib.SetMaxTemperature
SetMaxTemperature.restype = c_int
SetMaxTemperature.argtypes = [c_int,c_int,c_float]

GetMaxTemperature = TC300Lib.GetMaxTemperature
GetMaxTemperature.restype = c_int
GetMaxTemperature.argtypes = [c_int,c_int,POINTER(c_float)]

SetMinTemperature = TC300Lib.SetMinTemperature
SetMinTemperature.restype = c_int
SetMinTemperature.argtypes = [c_int,c_int,c_float]

GetMinTemperature = TC300Lib.GetMinTemperature
GetMinTemperature.restype = c_int
GetMinTemperature.argtypes = [c_int,c_int,POINTER(c_float)]

GetOutputVoltage = TC300Lib.GetOutputVoltage
GetOutputVoltage.restype = c_int
GetOutputVoltage.argtypes = [c_int,c_int,POINTER(c_float)]

GetTargetOutputCurrent = TC300Lib.GetTargetOutputCurrent
GetTargetOutputCurrent.restype = c_int
GetTargetOutputCurrent.argtypes = [c_int,c_int,POINTER(c_float)]

GetOutputCurrent = TC300Lib.GetOutputCurrent
GetOutputCurrent.restype = c_int
GetOutputCurrent.argtypes = [c_int,c_int,POINTER(c_float)]

SetOutputCurrent = TC300Lib.SetOutputCurrent
SetOutputCurrent.restype = c_int
SetOutputCurrent.argtypes = [c_int,c_int,c_float]

GetMaxVoltage = TC300Lib.GetMaxVoltage
GetMaxVoltage.restype = c_int
GetMaxVoltage.argtypes = [c_int,c_int,POINTER(c_float)]

SetMaxVoltage = TC300Lib.SetMaxVoltage
SetMaxVoltage.restype = c_int
SetMaxVoltage.argtypes = [c_int,c_int,c_float]

GetMaxCurrent = TC300Lib.GetMaxCurrent
GetMaxCurrent.restype = c_int
GetMaxCurrent.argtypes = [c_int,c_int,POINTER(c_float)]

SetMaxCurrent = TC300Lib.SetMaxCurrent
SetMaxCurrent.restype = c_int
SetMaxCurrent.argtypes = [c_int,c_int,c_float]

GetMode = TC300Lib.GetMode
GetMode.restype = c_int
GetMode.argtypes = [c_int,c_int,POINTER(c_int)]

SetMode = TC300Lib.SetMode
SetMode.restype = c_int
SetMode.argtypes = [c_int,c_int,c_int]

GetSensorOffset = TC300Lib.GetSensorOffset
GetSensorOffset.restype = c_int
GetSensorOffset.argtypes = [c_int,c_int,POINTER(c_float)]

SetSensorOffset = TC300Lib.SetSensorOffset
SetSensorOffset.restype = c_int
SetSensorOffset.argtypes = [c_int,c_int,c_float]

GetSensorType = TC300Lib.GetSensorType
GetSensorType.restype = c_int
GetSensorType.argtypes = [c_int,c_int,POINTER(c_int)]

SetSensorType = TC300Lib.SetSensorType
SetSensorType.restype = c_int
SetSensorType.argtypes = [c_int,c_int,c_int]

GetSensorParameter = TC300Lib.GetSensorParameter
GetSensorParameter.restype = c_int
GetSensorParameter.argtypes = [c_int,c_int,POINTER(c_int)]

SetSensorParameter = TC300Lib.SetSensorParameter
SetSensorParameter.restype = c_int
SetSensorParameter.argtypes = [c_int,c_int,c_int]

GetNtcBeta = TC300Lib.GetNtcBeta
GetNtcBeta.restype = c_int
GetNtcBeta.argtypes = [c_int,c_int,POINTER(c_float)]

SetNtcBeta = TC300Lib.SetNtcBeta
SetNtcBeta.restype = c_int
SetNtcBeta.argtypes = [c_int,c_int,c_float]

GetExtBeta = TC300Lib.GetExtBeta
GetExtBeta.restype = c_int
GetExtBeta.argtypes = [c_int,c_int,POINTER(c_float)]

SetExtBeta = TC300Lib.SetExtBeta
SetExtBeta.restype = c_int
SetExtBeta.argtypes = [c_int,c_int,c_float]





GetTriggerMode = TC300Lib.GetTriggerMode
GetTriggerMode.restype = c_int
GetTriggerMode.argtypes = [c_int,c_int,POINTER(c_int)]

SetTriggerMode = TC300Lib.SetTriggerMode
SetTriggerMode.restype = c_int
SetTriggerMode.argtypes = [c_int,c_int,c_int]

GetBrightness = TC300Lib.GetBrightness
GetBrightness.restype = c_int
GetBrightness.argtypes = [c_int,POINTER(c_int)]

SetBrightness = TC300Lib.SetBrightness
SetBrightness.restype = c_int
SetBrightness.argtypes = [c_int,c_int]

GetKnobState = TC300Lib.GetKnobState
GetKnobState.restype = c_int
GetKnobState.argtypes = [c_int,POINTER(c_int)]

SetKnobState = TC300Lib.SetKnobState
SetKnobState.restype = c_int
SetKnobState.argtypes = [c_int,c_int]

GetPIDParameter = TC300Lib.GetPIDParameter
GetPIDParameter.restype = c_int
GetPIDParameter.argtypes = [c_int,c_int,POINTER(c_float),POINTER(c_float),POINTER(c_float),POINTER(c_int)]

GetPIDParameterP = TC300Lib.GetPIDParameterP
GetPIDParameterP.restype = c_int
GetPIDParameterP.argtypes = [c_int,c_int,POINTER(c_float)]

SetPIDParameterP = TC300Lib.SetPIDParameterP
SetPIDParameterP.restype = c_int
SetPIDParameterP.argtypes = [c_int,c_int,c_float]

GetPIDParameterI = TC300Lib.GetPIDParameterI
GetPIDParameterI.restype = c_int
GetPIDParameterI.argtypes = [c_int,c_int,POINTER(c_float)]

SetPIDParameterI = TC300Lib.SetPIDParameterI
SetPIDParameterI.restype = c_int
SetPIDParameterI.argtypes = [c_int,c_int,c_float]

GetPIDParameterD = TC300Lib.GetPIDParameterD
GetPIDParameterD.restype = c_int
GetPIDParameterD.argtypes = [c_int,c_int,POINTER(c_float)]

SetPIDParameterD = TC300Lib.SetPIDParameterD
SetPIDParameterD.restype = c_int
SetPIDParameterD.argtypes = [c_int,c_int,c_float]

GetPIDParameterPeriod = TC300Lib.GetPIDParameterPeriod
GetPIDParameterPeriod.restype = c_int
GetPIDParameterPeriod.argtypes = [c_int,c_int,POINTER(c_int)]

SetPIDParameterPeriod = TC300Lib.SetPIDParameterPeriod
SetPIDParameterPeriod.restype = c_int
SetPIDParameterPeriod.argtypes = [c_int,c_int,c_int]

LoadDefaultPIDParameter = TC300Lib.LoadDefaultPIDParameter
LoadDefaultPIDParameter.restype = c_int
LoadDefaultPIDParameter.argtypes = [c_int,c_int]

GetMonitorMessage = TC300Lib.GetMonitorMessage
GetMonitorMessage.restype = c_int
GetMonitorMessage.argtypes = [c_int,POINTER(c_float),POINTER(c_float),POINTER(c_float),POINTER(c_float),POINTER(c_float),POINTER(c_float),POINTER(c_float),POINTER(c_float),POINTER(c_int),POINTER(c_int),POINTER(c_int),
                              POINTER(c_float),POINTER(c_float),POINTER(c_float),POINTER(c_float),POINTER(c_float),POINTER(c_float),POINTER(c_float),POINTER(c_float),POINTER(c_int),POINTER(c_int),POINTER(c_int)]

GetErrorMessage = TC300Lib.GetErrorMessage
GetErrorMessage.restype = c_int
GetErrorMessage.argtypes = [c_int,POINTER(c_ubyte)]

GetWarningMessage = TC300Lib.GetWarningMessage
GetWarningMessage.restype = c_int
GetWarningMessage.argtypes = [c_int,POINTER(c_ubyte)]

LoadFactoryParameter = TC300Lib.LoadFactoryParameter
LoadFactoryParameter.restype = c_int
LoadFactoryParameter.argtypes = [c_int]

#region command for TC300
def TC300ListDevices():
    """ List all connected TC300 devices
    Returns: 
       The TC300 device list, each deice item is [serialNumber]
    """
    str = create_string_buffer(1024, '\0') 
    result = List(str,1024)
    devicesStr = str.raw.decode("utf-8").rstrip('\x00').split(',')
    length = len(devicesStr)
    i = 0
    devices = []
    devInfo = ["",""]
    while(i < length):
        str = devicesStr[i]
        if (i % 2 == 0):
            if str != '':
                devInfo[0] = str
            else:
                i+=1
        else:
            devInfo[1] = str
            devices.append(devInfo.copy())
        i+=1
    return devices


def TC300Open(serialNo, nBaud, timeout):
    """ Open TC300 device
    Args:
        serialNo: serial number of TC300 device
        nBaud: bit per second of port
        timeout: set timeout value in (s)
    Returns: 
        non-negative number: hdl number returned Successful; negative number: failed.
    """
    return Open(serialNo.encode('utf-8'), nBaud, timeout)

def TC300IsOpen(serialNo):
    """ Check opened status of TC300 device
    Args:
        serialNo: serial number of TC300 device
    Returns: 
        0: TC300 device is not opened; 1: TC300 device is opened.
    """
    return IsOpen(serialNo.encode('utf-8'))

def TC300GetHandle(serialNo):
    """ get handle of port
    Args:
        serialNo: serial number of the device to be checked.
    Returns: 
        -1:no handle  non-negtive number: handle.
    """
    return GetHandle(serialNo.encode('utf-8'))

def TC300Close(hdl):
    """ Close opened TC300 device
    Args:
        hdl: the handle of opened TC300 device
    Returns: 
        0: Success; negative number: failed.
    """
    return Close(hdl)

def TC300GetId(hdl, id):
    """ get the TC300 id
    Args:
        hdl: the handle of opened TC300 device
        value: the model number, hardware and firmware versions
    Returns: 
        0: Success; negative number: failed.
    """
    idStr = create_string_buffer(1024,'\0')
    ret = GetId(hdl,idStr)
    id[0] = idStr.raw.decode("utf-8","ignore").rstrip('\x00').replace(">\r\n", "")
    return ret

def TC300GetStatus(hdl, status):
    """  Get device status
    Args:
        hdl: handle of port.
        status: 
          Device state define 	8bit
          bit0 = 0 or 1	chan1 disabled / enabled
          bit1 = 0 or 1	chan2 disabled / enabled
          bit2 = 1	normal
          bit3 = 1	warning
          bit4 = 1	error
          bit5	Reserved
          bit6	Reserved
          bit7	Reserved
    Returns: 
          0: Success; negative number: failed.
    """
    sta = c_ubyte(0) 
    ret = GetStatus(hdl,sta)
    status[0] = sta.value
    return ret

def TC300EnableChannel(hdl, channel, status):
    """ Set channel enable state 
    Args:
       hdl: handle of port.
       channel: channel index, 1 or 2.
       status: 0: Disable; 1: Enable
    Returns: 
        0: Success; negative number: failed.
    """
    return EnableChannel(hdl, channel, status)

def TC300GetChannels(hdl, channels):
    """  Get number of channels
    Args:
         hdl: handle of port.
         channels: channel number: 0: single channel; 1: dual channel.
    Returns: 
         0: Success; negative number: failed
    """
    val = c_int(0)
    ret = GetChannels(hdl,val)
    channels[0] = val.value
    return ret




def TC300CopyParameters(hdl):
    """ Copy channel 1 parameters to channel 2
    Args:
       hdl: handle of port.     
    Returns: 
        0: Success; negative number: failed.
    """
    return CopyParameters(hdl)




def TC300SetChannels(hdl, channels):
    """ Set number of channels
    Args:
        hdl: handle of port.
        channels: channel number: 0: single channel; 1: dual channel.
    Returns: 
        0: Success; negative number: failed.
    """
    return SetChannels(hdl, channels)

def TC300GetR0Constant(hdl, channel, r0):
    """  Get R0 value when sensor type is NTC1 or EXT1
    Args:
         hdl: handle of port.
         channel: channel index, 1 or 2.
         r0: the R0 value for NTC1 or EXT1 sensor.
    Returns: 
         0: Success; negative number: failed
    """
    val = c_float(0)
    ret = GetR0Constant(hdl,channel,val)
    r0[0] = val.value
    return ret

def TC300SetR0Constant(hdl, channel, r0):
    """ Set R0 value when sensor type is NTC1 or EXT1
    Args:
        hdl: handle of port.
        channel: channel index, 1 or 2.
        r0: the R0 value for NTC1 or EXT1 sensor 0~999.
    Returns: 
        0: Success; negative number: failed.
    """
    return SetR0Constant(hdl, channel, r0)

def TC300GetT0Constant(hdl, channel, t0):
    """  Get T0 value when sensor type is NTC1 or EXT1
    Args:
         hdl: handle of port.
         channel: channel index, 1 or 2.
         t0: the T0 value for NTC1 or EXT1 sensor.
    Returns: 
         0: Success; negative number: failed
    """
    val = c_float(0)
    ret = GetT0Constant(hdl,channel,val)
    t0[0] = val.value
    return ret

def TC300SetT0Constant(hdl, channel, t0):
    """ Set T0 value when sensor type is NTC1 or EXT1
    Args:
        hdl: handle of port.
        channel: channel index, 1 or 2.
        t0: the T0 value for NTC1 or EXT1 sensor 0~999.
    Returns: 
        0: Success; negative number: failed.
    """
    return SetT0Constant(hdl, channel, t0)

def TC300GetHartAConstant(hdl, channel, hartA):
    """  Get Hart A value when sensor type is NTC2 or EXT2
    Args:
         hdl: handle of port.
         channel: channel index, 1 or 2.
         hartA: the Hart A value for NTC2 or EXT2 sensor.
    Returns: 
         0: Success; negative number: failed
    """
    val = c_float(0)
    ret = GetHartAConstant(hdl,channel,val)
    hartA[0] = val.value
    return ret

def TC300SetHartAConstant(hdl, channel, hartA):
    """ Set Hart A value when sensor type is NTC2 or EXT2
    Args:
        hdl: handle of port.
        channel: channel index, 1 or 2.
        hartA: the Hart A value for NTC2 or EXT2 sensor -9.9999~9.9999.
    Returns: 
        0: Success; negative number: failed.
    """
    return SetHartAConstant(hdl, channel, hartA)

def TC300GetHartBConstant(hdl, channel, hartB):
    """  Get Hart B value when sensor type is NTC2 or EXT2
    Args:
         hdl: handle of port.
         channel: channel index, 1 or 2.
         hartB: the Hart B value for NTC2 or EXT2 sensor.
    Returns: 
         0: Success; negative number: failed
    """
    val = c_float(0)
    ret = GetHartBConstant(hdl,channel,val)
    hartB[0] = val.value
    return ret

def TC300SetHartBConstant(hdl, channel, hartB):
    """ Set Hart B value when sensor type is NTC2 or EXT2
    Args:
        hdl: handle of port.
        channel: channel index, 1 or 2.
        hartB: the Hart B value for NTC2 or EXT2 sensor -9.9999~9.9999.
    Returns: 
        0: Success; negative number: failed.
    """
    return SetHartBConstant(hdl, channel, hartB)

def TC300GetHartCConstant(hdl, channel, hartC):
    """  Get Hart C value when sensor type is NTC2 or EXT2
    Args:
         hdl: handle of port.
         channel: channel index, 1 or 2.
         hartC: the Hart C value for NTC2 or EXT2 sensor.
    Returns: 
         0: Success; negative number: failed
    """
    val = c_float(0)
    ret = GetHartCConstant(hdl,channel,val)
    hartC[0] = val.value
    return ret

def TC300SetHartCConstant(hdl, channel, hartC):
    """ Set Hart C value when sensor type is NTC2 or EXT2
    Args:
        hdl: handle of port.
        channel: channel index, 1 or 2.
        hartC: the Hart C value for NTC2 or EXT2 sensor -9.9999~9.9999.
    Returns: 
        0: Success; negative number: failed.
    """
    return SetHartCConstant(hdl, channel, hartC)

def TC300GetDarkStatus(hdl, status):
    """ Get dark mode
    Args:
         hdl: handle of port.
         status: dark mode: 0: disable; 1: enable.
    Returns: 
         0: Success; negative number: failed
    """
    val = c_int(0)
    ret = GetDarkStatus(hdl,val)
    status[0] = val.value
    return ret

def TC300SetDarkStatus(hdl, status):
    """ Set dark mode
    Args:
        hdl: handle of port.
        status: dark mode: 0: disable; 1: enable.
    Returns: 
        0: Success; negative number: failed.
    """
    return SetDarkStatus(hdl, status)

def TC300GetQuietMode(hdl, mode):
    """ Get quiet mode
    Args:
         hdl: handle of port.
         mode: quiet mode: 0: disable; 1: enable.
    Returns: 
         0: Success; negative number: failed
    """
    val = c_int(0)
    ret = GetQuietMode(hdl,val)
    mode[0] = val.value
    return ret

def TC300SetQuietMode(hdl, mode):
    """ Set quiet mode
    Args:
        hdl: handle of port.
        mode: quiet mode: 0: disable; 1: enable.
    Returns: 
        0: Success; negative number: failed.
    """
    return SetQuietMode(hdl, mode)


def TC300GetActualTemperature(hdl, channel, temperature):
    """  get actual temperature
    Args:
         hdl: handle of port.
         channel: channel index, 1 or 2.
         temperature: the actual temperature.
    Returns: 
         0: Success; negative number: failed
    """
    val = c_float(0)
    ret = GetActualTemperature(hdl,channel,val)
    temperature[0] = val.value
    return ret

def TC300SetTargetTemperature(hdl, channel, temperature):
    """ Set target temperature
    Args:
        hdl: handle of port.
        channel: channel index, 1 or 2.
        temperature: the target temperature -200~400 °C.
    Returns: 
        0: Success; negative number: failed.
    """
    return SetTargetTemperature(hdl, channel, temperature)

def TC300GetTargetTemperature(hdl, channel, temperature):
    """  Get target temperature
    Args:
         hdl: handle of port.
         channel: channel index, 1 or 2.
         temperature: the target temperature.
    Returns: 
         0: Success; negative number: failed
    """
    val = c_float(0)
    ret = GetTargetTemperature(hdl,channel,val)
    temperature[0] = val.value
    return ret

def TC300SetMaxTemperature(hdl, channel, temperature):
    """ Set max temperature
    Args:
        hdl: handle of port.
        channel: channel index, 1 or 2.
        temperature: the max value of temperature: min temperature ~ 400 °C.
    Returns: 
        0: Success; negative number: failed.
    """
    return SetMaxTemperature(hdl, channel, temperature)

def TC300GetMaxTemperature(hdl, channel, temperature):
    """  Get max temperature
    Args:
         hdl: handle of port.
         channel: channel index, 1 or 2.
         temperature: the max value of temperature.
    Returns: 
         0: Success; negative number: failed
    """
    val = c_float(0)
    ret = GetMaxTemperature(hdl,channel,val)
    temperature[0] = val.value
    return ret

def TC300SetMinTemperature(hdl, channel, temperature):
    """ Set min temperature
    Args:
        hdl: handle of port.
        channel: channel index, 1 or 2.
        temperature: the min value of temperature: -200 ~ max temperature °C.
    Returns: 
        0: Success; negative number: failed.
    """
    return SetMinTemperature(hdl, channel, temperature)

def TC300GetMinTemperature(hdl, channel, temperature):
    """  get min temperature
    Args:
         hdl: handle of port.
         channel: channel index, 1 or 2.
         temperature: the min temperature.
    Returns: 
         0: Success; negative number: failed
    """
    val = c_float(0)
    ret = GetMinTemperature(hdl,channel,val)
    temperature[0] = val.value
    return ret

def TC300GetOutputVoltage(hdl, channel, voltage):
    """  Get output voltage
    Args:
         hdl: handle of port.
         channel: channel index, 1 or 2.
         voltage: the output voltage.
    Returns: 
         0: Success; negative number: failed
    """
    val = c_float(0)
    ret = GetOutputVoltage(hdl,channel,val)
    voltage[0] = val.value
    return ret

def TC300GetTargetOutputCurrent(hdl, channel, current):
    """  Get target output current
    Args:
         hdl: handle of port.
         channel: channel index, 1 or 2.
         current: the target output current.
    Returns: 
         0: Success; negative number: failed
    """
    val = c_float(0)
    ret = GetTargetOutputCurrent(hdl,channel,val)
    current[0] = val.value
    return ret

def TC300GetOutputCurrent(hdl, channel, current):
    """  Get actual output current
    Args:
         hdl: handle of port.
         channel: channel index, 1 or 2.
         current: the actual output current.
    Returns: 
         0: Success; negative number: failed
    """
    val = c_float(0)
    ret = GetOutputCurrent(hdl,channel,val)
    current[0] = val.value
    return ret

def TC300SetOutputCurrent(hdl, channel, current):
    """ Set target output current
    Args:
        hdl: handle of port.
        channel: channel index, 1 or 2.
        current: the target output current -2000~2000mA.
    Returns: 
        0: Success; negative number: failed.
    """
    return SetOutputCurrent(hdl, channel, current)

def TC300GetMaxVoltage(hdl, channel, voltage):
    """  Get max voltage
    Args:
         hdl: handle of port.
         channel: channel index, 1 or 2.
         voltage: the max value of voltage.
    Returns: 
         0: Success; negative number: failed
    """
    val = c_float(0)
    ret = GetMaxVoltage(hdl,channel,val)
    voltage[0] = val.value
    return ret

def TC300SetMaxVoltage(hdl, channel, voltage):
    """ Set max voltage
    Args:
        hdl: handle of port. 
        channel: channel index, 1 or 2.
        voltage: the max value of voltage 0.1 ~ 24.0 V.
    Returns: 
        0: Success; negative number: failed.
    """
    return SetMaxVoltage(hdl, channel, voltage)

def TC300GetMaxCurrent(hdl, channel, current):
    """  Get max current
    Args:
         hdl: handle of port.
         channel: channel index, 1 or 2.
         current: the max value of current(A).
    Returns: 
         0: Success; negative number: failed
    """
    val = c_float(0)
    ret = GetMaxCurrent(hdl,channel,val)
    current[0] = val.value
    return ret

def TC300SetMaxCurrent(hdl, channel, current):
    """ Set max current
    Args:
        hdl: handle of port.
        channel: channel index, 1 or 2.
        current: the max value of current 0~2000 mA.
    Returns: 
        0: Success; negative number: failed.
    """
    return SetMaxCurrent(hdl, channel, current)

def TC300GetMode(hdl, channel, mode):
    """  Get channel mode
    Args:
         hdl: handle of port.
         channel: channel index, 1 or 2.
         mode: channel mode: 0: Heater; 1: Tec; 2: Constant current; 3: Synchronize with Ch1(only for channel 2).
    Returns: 
         0: Success; negative number: failed
    """
    val = c_int(0)
    ret = GetMode(hdl,channel,val)
    mode[0] = val.value
    return ret

def TC300SetMode(hdl, channel, mode):
    """ Set channel mode
    Args:
        hdl: handle of port.
        channel: channel index, 1 or 2.
        mode: channel mode: 0: Heater; 1: Tec; 2: Constant current; 3: Synchronize with Ch1(only for channel 2).
    Returns: 
        0: Success; negative number: failed.
    """
    return SetMode(hdl, channel, mode)

def TC300GetSensorOffset(hdl, channel, offset):
    """  Get sensor offset when sensor type is PT100 or PT1000
    Args:
         hdl: handle of port.
         channel: channel index, 1 or 2.
         offset: the sensor offset
    Returns: 
         0: Success; negative number: failed
    """
    val = c_float(0)
    ret = GetSensorOffset(hdl,channel,val)
    offset[0] = val.value
    return ret

def TC300SetSensorOffset(hdl, channel, offset):
    """ Set sensor offset when sensor type is PT100 or PT1000
    Args:
        hdl: handle of port.
        channel: channel index, 1 or 2.
        offset: the sensor offset -10~10 °C
    Returns: 
        0: Success; negative number: failed.
    """
    return SetSensorOffset(hdl, channel, offset)

def TC300GetSensorType(hdl, channel, sensorType):
    """  Get sensor type
    Args:
         hdl: handle of port.
         channel: channel index, 1 or 2.
         sensorType: sensor type: 0: PT100; 1: PT1000; 2: NTC1; 3: NTC2; 4: Thermo; 5: AD590; 6: EXT1; 7: EXT2.
    Returns: 
         0: Success; negative number: failed
    """
    val = c_int(0)
    ret = GetSensorType(hdl,channel,val)
    sensorType[0] = val.value
    return ret

def TC300SetSensorType(hdl, channel, sensorType):
    """ Set channel mode
    Args:
        hdl: handle of port.
        channel: channel index, 1 or 2.
        sensorType: sensor type: 0: PT100; 1: PT1000; 2: NTC1; 3: NTC2; 4: Thermo; 5: AD590; 6: EXT1; 7: EXT2.
    Returns: 
        0: Success; negative number: failed.
    """
    return SetSensorType(hdl, channel, sensorType)

def TC300GetSensorParameter(hdl, channel, parameter):
    """  Get sensor parameter when sensor type is PT100
    Args:
         hdl: handle of port.
         channel: channel index, 1 or 2.
         parameter: sensor parameter: 0: 2 wire; 1: 3 wire; 2: 4 wire; 3: J type; 4: K type.
    Returns: 
         0: Success; negative number: failed
    """
    val = c_int(0)
    ret = GetSensorParameter(hdl,channel,val)
    parameter[0] = val.value
    return ret

def TC300SetSensorParameter(hdl, channel, parameter):
    """ Set sensor parameter when sensor type is PT100
    Args:
        hdl: handle of port.
        channel: channel index, 1 or 2.
        parameter: sensor parameter: 0: 2 wire; 1: 3 wire; 2: 4 wire; 3: J type; 4: K type.
    Returns: 
        0: Success; negative number: failed.
    """
    return SetSensorParameter(hdl, channel, parameter)


def TC300GetNtcBeta(hdl, channel, constant):
    """  Get β value when sensor type is ntc1
    args:
         hdl: handle of port.
         channel: channel index, 1 or 2.
         constant: the β value for ntc1 sensor.
    returns: 
         0: success; negative number: failed
    """
    val = c_float(0)
    ret = GetNtcBeta(hdl,channel,val)
    constant[0] = val.value
    return ret

def TC300SetNtcBeta(hdl, channel, constant):
    """ Set β value when sensor type is ntc1
    args:
        hdl: handle of port.
        channel: channel index, 1 or 2.
        constant: the β value for ntc1 sensor 0~9999.
    returns: 
        0: success; negative number: failed.
    """
    return SetNtcBeta(hdl, channel, constant)

def TC300GetExtBeta(hdl, channel, constant):
    """  Get β value when sensor type is EXT1
    args:
         hdl: handle of port.
         channel: channel index, 1 or 2.
         constant: the β value for EXT1 sensor.
    returns: 
         0: success; negative number: failed
    """
    val = c_float(0)
    ret = GetExtBeta(hdl,channel,val)
    constant[0] = val.value
    return ret

def TC300SetExtBeta(hdl, channel, constant):
    """ Set β value when sensor type is EXT1
    args:
        hdl: handle of port.
        channel: channel index, 1 or 2.
        constant: the β value for EXT1 sensor 0~9999.
    returns: 
        0: success; negative number: failed.
    """
    return SetExtBeta(hdl, channel, constant)




def TC300GetTriggerMode(hdl, channel, mode):
    """  Get trigger mode
    Args:
         hdl: handle of port.
         channel: channel index, 1 or 2.
         mode: trigger mode: 0: output; 1: input.
    Returns: 
         0: Success; negative number: failed
    """
    val = c_int(0)
    ret = GetTriggerMode(hdl,channel,val)
    mode[0] = val.value
    return ret

def TC300SetTriggerMode(hdl, channel, mode):
    """ Set trigger mode
    Args:
        hdl: handle of port.
        channel: channel index, 1 or 2.
        mode: trigger mode: 0: output; 1: input.
    Returns: 
        0: Success; negative number: failed.
    """
    return SetTriggerMode(hdl, channel, mode)

def TC300GetBrightness(hdl, brightness):
    """  Get LCD brightness
    Args:
         hdl: handle of port.
         brightness: LCD brightness
    Returns: 
         0: Success; negative number: failed
    """
    val = c_int(0)
    ret = GetBrightness(hdl, val)
    brightness[0] = val.value
    return ret

def TC300SetBrightness(hdl, brightness):
    """ Set LCD brightness
    Args:
        hdl: handle of port.        
        brightness: LCD brightness 10~100.
    Returns: 
        0: Success; negative number: failed.
    """
    return SetBrightness(hdl, brightness)

def TC300GetKnobState(hdl, state):
    """  Get knob state
    Args:
         hdl: handle of port.
         state: knob state: 0: unlocked; 1: locked.
    Returns: 
         0: Success; negative number: failed
    """
    val = c_int(0)
    ret = GetKnobState(hdl,val)
    state[0] = val.value
    return ret

def TC300SetKnobState(hdl, state):
    """ Set knob state
    Args:
        hdl: handle of port.
        state: knob state: 0: unlocked; 1: locked.
    Returns: 
        0: Success; negative number: failed.
    """
    return SetKnobState(hdl, state)

def TC300GetPIDParameter(hdl, channel, p, i, d, period):
    """  Get PID parameters
    Args:
         hdl: handle of port.
         channel: channel index, 1 or 2.
         p: Kp
         i: Ti
         d: Td
         period: period time
    Returns: 
         0: Success; negative number: failed
    """
    kp = c_float(0)
    ti = c_float(0)
    td = c_float(0)
    peri = c_int(0)
    ret = GetPIDParameter(hdl,channel,kp,ti,td,peri)
    p[0] = kp.value
    i[0] = ti.value
    d[0] = td.value
    period[0] = peri.value
    return ret

def TC300GetPIDParameterP(hdl, channel, p):
    """  PID parameter Kp
    Args:
         hdl: handle of port.
         channel: channel index, 1 or 2.
         p: the PID parameter Kp 
    Returns: 
         0: Success; negative number: failed
    """
    val = c_float(0)
    ret = GetPIDParameterP(hdl,channel,val)
    p[0] = val.value
    return ret

def TC300SetPIDParameterP(hdl, channel, p):
    """ Set PID parameter Kp
    Args:
        hdl: handle of port.
        channel: channel index, 1 or 2.
        p: the PID parameter Kp 0~9.99 A/K
    Returns: 
        0: Success; negative number: failed.
    """
    return SetPIDParameterP(hdl, channel, p)

def TC300GetPIDParameterI(hdl, channel, i):
    """  PID parameter Ti
    Args:
         hdl: handle of port.
         channel: channel index, 1 or 2.
         i: the PID parameter Ki 
    Returns: 
         0: Success; negative number: failed
    """
    val = c_float(0)
    ret = GetPIDParameterI(hdl,channel,val)
    i[0] = val.value
    return ret

def TC300SetPIDParameterI(hdl, channel, i):
    """ Set PID parameter Ti
    Args:
        hdl: handle of port.
        channel: channel index, 1 or 2.
        i: the PID parameter Ti 0~9.99 A/(K*sec)
    Returns: 
        0: Success; negative number: failed.
    """
    return SetPIDParameterI(hdl, channel, i)

def TC300GetPIDParameterD(hdl, channel, d):
    """  PID parameter Td
    Args:
         hdl: handle of port.
         channel: channel index, 1 or 2.
         i: the PID parameter Kd 
    Returns: 
         0: Success; negative number: failed
    """
    val = c_float(0)
    ret = GetPIDParameterD(hdl,channel,val)
    d[0] = val.value
    return ret

def TC300SetPIDParameterD(hdl, channel, d):
    """ Set PID parameter Td
    Args:
        hdl: handle of port.
        channel: channel index, 1 or 2.
        d: the PID parameter Td 0~9.99 (A*sec)/K
    Returns: 
        0: Success; negative number: failed.
    """
    return SetPIDParameterD(hdl, channel, d)

def TC300GetPIDParameterPeriod(hdl, channel, period):
    """  Get PID period time
    Args:
         hdl: handle of port.
         channel: channel index, 1 or 2.
         period: the PID period time
    Returns: 
         0: Success; negative number: failed
    """
    val = c_int(0)
    ret = GetPIDParameterPeriod(hdl,channel,val)
    period[0] = val.value
    return ret

def TC300SetPIDParameterPeriod(hdl, channel, period):
    """ Set PID period time
    Args:
        hdl: handle of port.
        channel: channel index, 1 or 2.
        period: the PID period time 100~5000 ms
    Returns: 
        0: Success; negative number: failed.
    """
    return SetPIDParameterPeriod(hdl, channel, period)

def TC300LoadDefaultPIDParameter(hdl, channel):
    """ Reset PID parameters to default
    Args:
        hdl: handle of port.
        channel: channel index, 1 or 2.
    Returns: 
        0: Success; negative number: failed.
    """
    return LoadDefaultPIDParameter(hdl, channel)

def TC300GetMonitorMessage(hdl, ch1TargTemp, ch1Temp, ch1TargCurr, ch1Curr, ch1MinTemp, ch1MaxTemp, ch1MaxCurr, ch1Volt, ch1Mode, ch1Trig, ch1SensType,
	ch2TargTemp, ch2Temp, ch2TargCurr, ch2Curr, ch2MinTemp, ch2MaxTemp, ch2MaxCurr,ch2Volt, ch2Mode, ch2Trig, ch2SensType):
    """  Get all status in one command
    Args:
         hdl: handle of port.
         ch1TargTemp: channel 1 target temperature.
         ch1Temp: channel 1 actual temperature.
         ch1TargCurr: channel 1 target current.
         ch1Curr: channel 1 current.
         ch1MinTemp: channel 1 minimum temperature.
         ch1MaxTemp: channel 1 maximum temperature.
         ch1MaxCurr: channel 1 maximum current.
         ch1Volt: channel 1 voltage.
         ch1Mode: channel 1 mode: 0: Heater; 1: Tec; 2: Constant current;.
         ch1Trig: channel 1 trigger mode: 0: output; 1: input.
         ch1SensType: channel 1 sensor type: 0: PT100; 1: PT1000; 2: NTC10K; 3: Thermo; 4: AD590; 5: Ext TSP-TH.
         ch2TargTemp: channel 2 target temperature.
         ch2Temp: channel 2 actual temperature.
         ch2TargCurr: channel 2 target current.
         ch2Curr: channel 2 current.
         ch2MinTemp: channel 2 minimum temperature.
         ch2MaxTemp: channel 2 maximum temperature.
         ch2MaxCurr: channel 2 maximum current.
         ch2Volt: channel 2 voltage.
         ch2Mode: channel 2 mode: 0: Heater; 1: Tec; 2: Constant current; 3: Synchronize with Ch1.
         ch2Trig: channel 2 trigger mode: 0: output; 1: input.
         ch2SensType: channel 2 sensor type: 0: PT100; 1: PT1000; 2: NTC10K; 3: Thermo; 4: AD590; 5: Ext TSP-TH.
    Returns: 
         0: Success; negative number: failed
    """    
    ch1TargTempval = c_float(0)
    ch1Tempval = c_float(0)
    ch1TargCurrval = c_float(0)
    ch1Currval = c_float(0)
    ch1MinTempval = c_float(0)
    ch1MaxTempval = c_float(0)
    ch1MaxCurrval = c_float(0)
    ch1Voltval = c_float(0)
    ch1Modeval = c_int(0)
    ch1Trigval = c_int(0)
    ch1SensTypeval = c_int(0)
    ch2TargTempval = c_float(0)
    ch2Tempval = c_float(0)
    ch2TargCurrval = c_float(0)
    ch2Currval = c_float(0)
    ch2MinTempval = c_float(0)
    ch2MaxTempval = c_float(0)
    ch2MaxCurrval = c_float(0)
    ch2Voltval = c_float(0)
    ch2Modeval = c_int(0)
    ch2Trigval = c_int(0)
    ch2SensTypeval = c_int(0)
    ret = GetMonitorMessage(hdl, ch1TargTempval, ch1Tempval,ch1TargCurrval, ch1Currval,ch1MinTempval,ch1MaxTempval,ch1MaxCurrval, ch1Voltval, ch1Modeval, ch1Trigval, ch1SensTypeval, 
                            ch2TargTempval, ch2Tempval, ch2TargCurrval, ch2Currval, ch2MinTempval, ch2MaxTempval, ch2MaxCurrval, ch2Voltval, ch2Modeval, ch2Trigval, ch2SensTypeval)
    ch1TargTemp[0] = ch1TargTempval.value
    ch1Temp[0] = ch1Tempval.value
    ch1TargCurr[0]=ch1TargCurrval.value
    ch1Curr[0] = ch1Currval.value
    ch1MinTemp[0]=ch1MinTempval.value
    ch1MaxTemp[0]=ch1MaxTempval.value
    ch1MaxCurr[0]=ch1MaxCurrval.value
    ch1Volt[0] = ch1Voltval.value
    ch1Mode[0] = ch1Modeval.value
    ch1Trig[0] = ch1Trigval.value
    ch1SensType[0] = ch1SensTypeval.value
    ch2TargTemp[0] = ch2TargTempval.value
    ch2Temp[0] = ch2Tempval.value
    ch2TargCurr[0]=ch2TargCurrval.value
    ch2Curr[0] = ch2Currval.value
    ch2MinTemp[0] = ch2MinTempval.value
    ch2MaxTemp[0] = ch2MaxTempval.value
    ch2MaxCurr[0] = ch2MaxCurrval.value
    ch2Volt[0] = ch2Voltval.value
    ch2Mode[0] = ch2Modeval.value
    ch2Trig[0] = ch2Trigval.value
    ch2SensType[0] = ch2SensTypeval.value
    return ret

def TC300GetErrorMessage(hdl, message):
    """  Get error message when GetStatus result contains an error
    Args:
         hdl: handle of port.
         message: 
            Device state define 	8bit
            bit0 = 1	No Load Ch1
            bit1 = 1	No Sensor Ch1
            bit2 = 1	No Load Ch2
            bit3 = 1	No Sensor Ch2
            bit4 = 1	Future definition
            bit5 = 1	Future definition
            bit6 = 1	Future definition
            bit7 = 1	Hsink Temper High
    Returns: 
         0: Success; negative number: failed
    """
    val = c_ubyte(0)
    ret = GetErrorMessage(hdl,val)
    message[0] = val.value
    return ret

def TC300GetWarningMessage(hdl, message):
    """  Get warning message when GetStatus result contains an error
    Args:
         hdl: handle of port.
         message: 
            Device state define 	8bit
            bit0 = 1	Amb Temper High
            bit1 = 1	Ch1 Temper High
            bit2 = 1	Ch1 Temper Low
            bit3 = 1	Ch2 Temper High
            bit4 = 1	Ch2 Temper Low
            bit5 = 1	Future definition
            bit6 = 1	Future definition
             bit7 = 1	Future definition
    Returns: 
         0: Success; negative number: failed
    """
    val = c_ubyte(0)
    ret = GetWarningMessage(hdl,val)
    message[0] = val.value
    return ret

def TC300LoadFactoryParameter(hdl):
    """ Restore factory settings
    Args:
        hdl: handle of port.
    Returns: 
        0: Success; negative number: failed.
    """
    return LoadFactoryParameter(hdl)

#endregion




import pyThorlabsTC300.TC300_COMMAND_LIB as TC300_COMMAND_LIB 

class ThorlabsTC300:


    def __init__(self):
        self.connected = False
  
        self.device_handle = None
        self.device_serial = None
     
    def list_devices(self):
        '''
        Scans all potential devices, ask for their identity and check if any of them is a valid device supported by this driver (by comparing their identity with the elements of model_identifiers)

        Returns
        -------
        list_valid_devices, list
            A list of all found valid devices. Each element of the list is a list of three strings, in the format [address,identity,model]

        '''

        self.list_all_devices = TC300_COMMAND_LIB.TC300ListDevices()
        self.list_valid_devices = self.list_all_devices
        # self.list_valid_devices = [] 
        # for addr in self.list_all_devices:
        #     if(not(addr.startswith('ASRL'))):
        #         try:
        #             instrument = self.rm.open_resource(addr)
        #             idn = instrument.query('*IDN?').strip()
        #             for model in self.model_identifiers: #sweep over all supported models
        #                 if model[1] in idn:              #check if idn is one of the supported models
        #                     if self.model_user  and not(self.model_user ==model[0]): #if the user had specified a specific model, we dont consider any other model
        #                         break
        #                     self.list_valid_devices.append([addr,idn,model[0]])
        #             instrument.before_close()
        #             instrument.close()     
        #         except visa.VisaIOError:
        #             pass
        return self.list_valid_devices
    
    def connect_device(self,device_addr):
        self.list_devices()
        device_addresses = [dev[0] for dev in self.list_valid_devices]
        if (device_addr in device_addresses):
            try:         
                hdl = TC300_COMMAND_LIB.TC300Open(device_addr,115200,3)    
                #hdl is less than 0 if something went wrong
                ID = 1
            except:
                Msg = "Error during connection"
                ID = -1 
        else:
            raise ValueError("The specified address is not a valid device address.")
        if(ID>=0):
            self.connected = True
            self.device_handle = hdl
            self.device_serial = device_addr
            Msg = 'Connected'
            #self.read_parameters_upon_connection()
        else:
            self.connected = False
            self.device_handle = None
            self.device_serial = None
            Msg = 'Error during connection'
        return (ID,Msg)

    def read_parameters_upon_connection(self):
        self.wavelength
        self.read_min_max_wavelength()
        self.power
        self.min_power_range
        self.max_power_range
        self.auto_power_range
        self.power_range

    def disconnect_device(self):
        if(self.connected == True):
            try:   
                TC300_COMMAND_LIB.TC300Close(self.device_handle)  
                ID = 1
                Msg = 'Disconnected'
            except Exception as e:
                ID = -1 
                Msg = e
            self.connected = False
            return (ID,Msg)
        else:
            raise RuntimeError("Device is already disconnected.")

    def enable_disable_channel(self,channel,desired_status):
        # desired_status =  0: Disable; 1: Enable
        result = TC300_COMMAND_LIB.TC300EnableChannel(self.device_handle,channel,desired_status)  
        if(result<0):
            raise RuntimeError(f"Error while enabling/disabling channel: {result}")
        else:
            return result
    
    def set_mode_channel(self,channel,mode):
        #mode: 0: Heater; 1: Tec; 2: Constant current; 3: Synchronize with Ch1(only for channel 2)
        result = TC300_COMMAND_LIB.TC300SetMode(self.device_handle, channel,mode) #channel mode: 0: Heater; 1: Tec; 2: Constant current; 3: Synchronize with Ch1(only for channel 2)
        if(result<0):
            raise RuntimeError(f"Error while setting the channel mode: {result}")
        else:
            return result

    def get_mode_channel(self,channel):
        #mode: 0: Heater; 1: Tec; 2: Constant current; 3: Synchronize with Ch1(only for channel 2)
        mode=[0]
        result = TC300_COMMAND_LIB.TC300GetMode(self.device_handle, channel,mode) #channel mode: 0: Heater; 1: Tec; 2: Constant current; 3: Synchronize with Ch1(only for channel 2)
        if(result<0):
            raise RuntimeError(f"Error while reading the channel mode: {result}")
        else:
            return mode[0]

    def get_actual_temperature(self,channel):
        ActualTemperature=[0]
        result = TC300_COMMAND_LIB.TC300GetActualTemperature(self.device_handle,channel,ActualTemperature)
        if(result<0):
            raise RuntimeError(f"Error while reading actual temperature: {result}")
        else:
            return ActualTemperature[0]

    def set_target_temperature(self,channel,T):
        result = TC300_COMMAND_LIB.TC300SetTargetTemperature(self.device_handle,channel,T) 
        if(result<0):
            raise RuntimeError(f"Error while setting target temperature: {result}")
        else:
            return result 
        
    def get_target_temperature(self,channel):
        TargetTemperature=[0]
        result = TC300_COMMAND_LIB.TC300GetTargetTemperature(self.device_handle,channel,TargetTemperature)
        if(result<0):
            raise RuntimeError(f"Error while reading target temperature: {result}")
        else:
            return TargetTemperature[0]  

    def get_actual_current(self,channel):
        ActualCurrent=[0]
        result = TC300_COMMAND_LIB.TC300GetOutputCurrent(self.device_handle,channel,ActualCurrent)
        if(result<0):
            raise RuntimeError(f"Error while reading actual current: {result}")
        else:
            return ActualCurrent[0]

    def set_target_current(self,channel,I):
        result = TC300_COMMAND_LIB.TC300SetOutputCurrent(self.device_handle,channel,I) 
        if(result<0):
            raise RuntimeError(f"Error while setting target current: {result}")
        else:
            return result 
        
    def get_target_current(self,channel):
        TargetCurrent=[0]
        result = TC300_COMMAND_LIB.TC300GetTargetOutputCurrent(self.device_handle,channel,TargetCurrent)
        if(result<0):
            raise RuntimeError(f"Error while reading target current: {result}")
        else:
            return TargetCurrent[0]  
        
    def read_min_max_temperature(self,channel):
        MinTemperature=[0]
        result = TC300_COMMAND_LIB.TC300GetMinTemperature(self.device_handle,channel,MinTemperature)
        if(result<0):
                raise RuntimeError(f"Error while reading min temperature: {result}")
        MaxTemperature=[0]
        result = TC300_COMMAND_LIB.TC300GetMaxTemperature(self.device_handle,channel,MaxTemperature)
        if(result<0):
                raise RuntimeError(f"Error while reading max temperature: {result}")
        return (MinTemperature[0],MaxTemperature[0])

    def set_min_max_temperature(self,channel,mT,MT):
        result = TC300_COMMAND_LIB.TC300SetMinTemperature(self.device_handle, channel, mT) 
        if(result<0):
            raise RuntimeError(f"Error while setting min temperature: {result}")
        result = TC300_COMMAND_LIB.TC300SetMaxTemperature(self.device_handle, channel, MT) 
        if(result<0):
            raise RuntimeError(f"Error while setting max temperature: {result}")
        return True

    def read_PID_parameters(self,channel):
        P=[0]
        result = TC300_COMMAND_LIB.TC300GetPIDParameterP(self.device_handle, channel,P)
        if(result<0):
                raise RuntimeError(f"Error while reading PID parameter P: {result}")
        I=[0]
        result = TC300_COMMAND_LIB.TC300GetPIDParameterI(self.device_handle, channel,I)
        if(result<0):
                raise RuntimeError(f"Error while reading PID parameter I: {result}")
        D=[0]
        result = TC300_COMMAND_LIB.TC300GetPIDParameterD(self.device_handle, channel,D)
        if(result<0):
                raise RuntimeError(f"Error while reading PID parameter D: {result}")
        Period=[0]
        result = TC300_COMMAND_LIB.TC300GetPIDParameterPeriod(self.device_handle, channel,Period)
        if(result<0):
                raise RuntimeError(f"Error while reading PID parameter Period: {result}")
        return (P[0],I[0],D[0],Period[0])
    
    def set_PID_parameter(self,channel,Name,Value):
        PossibleNames = ['P','I','D','Period']
        Functions =[TC300_COMMAND_LIB.TC300SetPIDParameterP,TC300_COMMAND_LIB.TC300SetPIDParameterI,TC300_COMMAND_LIB.TC300SetPIDParameterD,TC300_COMMAND_LIB.TC300SetPIDParameterPeriod]
        if Name in PossibleNames:
            fun = Functions[PossibleNames.index(Name)]
            result = fun(self.device_handle, channel, Value) 
        if(result<0):
            raise RuntimeError(f"Error while setting PID parameter {Name}: {result}")
        return True


    



try:
    from TC300_COMMAND_LIB import *
    import time
except OSError as ex:
    print("Warning:",ex)
         

# ------------ Example Channel Read&Write for channel 1-------------- # 
def ChannelReadWrite(hdl):
   print("*** Channel Read&Write example")   

   #set channel count to 2
   result = TC300SetChannels(hdl, 1)  #  channel number:0: single channel; 1: dual channel
   if(result<0):
       print("Set dual channel fail" , result)
   else:
       print("channel number is dual channel")

   channels=[0]
   channelsList={0:"single channel", 1:"dual channel"}
   result=TC300GetChannels(hdl,channels)
   if(result<0):
      print("Get channel number fail",result)
   else:
      print("Get channel number :",channelsList.get(channels[0]))

   #Enable channel 1
   result = TC300EnableChannel(hdl, 1, 1)  #  0: Disable; 1: Enable
   if(result<0):
       print("Set channel 1 enabled fail" , result)
   else:
       print("channel 1 enabled")

   result = TC300SetMode(hdl, 1, 2) #channel mode: 0: Heater; 1: Tec; 2: Constant current; 3: Synchronize with Ch1(only for channel 2)
   if(result<0):
       print("Set mode fail" , result)
   else:
       print("mode is" , "Constant current")

   mode=[0]
   modeList={0: "Heater", 1: "Tec", 2: "Constant current", 3: "Synchronize with Ch1(only for channel 2)"}
   result=TC300GetMode(hdl,1,mode)
   if(result<0):
      print("Get mode fail",result)
   else:
      print("Get mode :",modeList.get(mode[0]))

   result = TC300SetOutputCurrent(hdl, 1, 500)  #  the output current -2000~2000mA
   if(result<0):
       print("Set target Output Current fail" , result)
   else:
       print("Sensor target Output Current is" , 500)

   OutputCurrent=[0]
   result=TC300GetTargetOutputCurrent(hdl,1,OutputCurrent)
   if(result<0):
      print("Get target Output Current fail",result)
   else:
      print("target Output Current is :",OutputCurrent[0])

   result = TC300SetTargetTemperature(hdl, 1, 50)  #   the target temperature -200~400 °C
   if(result<0):
       print("Set Target Temperature fail" , result)
   else:
       print("Target Temperature is" , 50)

   TargetTemperature=[0]
   result=TC300GetTargetTemperature(hdl,1,TargetTemperature)
   if(result<0):
      print("Get Target Temperature fail",result)
   else:
      print("Target Temperature is :",TargetTemperature[0])

   ActualTemperature=[0]
   result=TC300GetActualTemperature(hdl,1,ActualTemperature)
   if(result<0):
      print("Get Actual Temperature fail",result)
   else:
      print("Actual Temperature is :",ActualTemperature[0])   

   # Set channel 2 mode to Synchronize with Ch1(only for channel 2)
   result = TC300SetMode(hdl, 2, 3) #channel mode: 0: Heater; 1: Tec; 2: Constant current; 3: Synchronize with Ch1(only for channel 2)
   if(result<0):
       print("Set mode fail" , result)
   else:
       print("mode is" , "Synchronize with Ch1(only for channel 2)")

   mode=[0]
   modeList={0: "Heater", 1: "Tec", 2: "Constant current", 3: "Synchronize with Ch1(only for channel 2)"}
   result=TC300GetMode(hdl,2,mode)
   if(result<0):
      print("Get mode fail",result)
   else:
      print("Get mode :",modeList.get(mode[0]))  


# ------------ Example channel Parameters Read&Write for channel 1  -------------- # 
def ChannelParametersReadWrite(hdl):
   print("*** Channel Parameters Read&Write example")

   result = TC300SetTriggerMode(hdl, 1, 0) #0:output,1:input
   if(result<0):
       print("Set Trigger Mode fail" , result)
   else:
       print("Trigger Mode is" , "output")

   triggermode=[0]
   triggermodeList={0:"output", 1:"input"}
   result=TC300GetTriggerMode(hdl, 1, triggermode)
   if(result<0):
      print("Get Trigger Mode fail",result)
   else:
      print("Get Trigger Mode :",triggermodeList.get(triggermode[0]))

   result = TC300SetMinTemperature(hdl, 1, 10)  # the min value of temperature -200~50 °C
   if(result<0):
       print("Set Min Temperature fail" , result)
   else:
       print("Sensor Min Temperature is" , 10)

   MinTemperature=[0]
   result=TC300GetMinTemperature(hdl,1,MinTemperature)
   if(result<0):
      print("Get Min Temperature fail",result)
   else:
      print("Min Temperature is :",MinTemperature[0])

   result = TC300SetMaxTemperature(hdl, 1, 300)  # the max value of temperature 50~400 °C
   if(result<0):
       print("Set Max Temperature fail" , result)
   else:
       print("Sensor Max Temperature is" , 300)

   MaxTemperature=[0]
   result=TC300GetMaxTemperature(hdl,1,MaxTemperature)
   if(result<0):
      print("Get Max Temperature fail",result)
   else:
      print("Max Temperature is :",MaxTemperature[0])

   result = TC300SetMaxCurrent(hdl, 1, 1500)  # the max value of current 0~2000 mA
   if(result<0):
       print("Set Max Current fail" , result)
   else:
       print("Sensor Max Current is" , 1500)

   MaxCurrent=[0]
   result=TC300GetMaxCurrent(hdl,1,MaxCurrent)
   if(result<0):
      print("Get Max Current fail",result)
   else:
      print("Max Current is :",MaxCurrent[0])

   result = TC300SetMaxVoltage(hdl, 1, 12.3)  #  the max value of voltage 0.1-24.0 V
   if(result<0):
       print("Set Max Voltage fail" , result)
   else:
       print("Max Voltage is" , 12.3)

   MaxVoltage=[0]
   result=TC300GetMaxVoltage(hdl,1,MaxVoltage)
   if(result<0):
      print("Get Max Voltage fail",result)
   else:
      print("Max Voltage is :",MaxVoltage[0])

   result = TC300SetQuietMode(hdl, 1) #0: disable; 1: enable
   if(result<0):
       print("Set quiet mode fail" , result)
   else:
       print("quiet mode is enable")

   mode=[0]
   modeList={0:"disable", 1:"enable"}
   result=TC300GetQuietMode(hdl, mode)
   if(result<0):
      print("Get quiet mode fail",result)
   else:
      print("Get quiet mode :",modeList.get(mode[0]))

   result = TC300SetDarkStatus(hdl, 0) #0: disable; 1: enable
   if(result<0):
       print("Set dark status fail" , result)
   else:
       print("dark status is disable")

   status=[0]
   statusList={0:"disable", 1:"enable"}
   result=TC300GetDarkStatus(hdl, status)
   if(result<0):
      print("Get dark status fail",result)
   else:
      print("Get dark status :",statusList.get(status[0]))           



# ------------ Example Sensor Type Parameters Read&Write for channel 1  -------------- # 
def SensorTypeParametersReadWrite(hdl):
   print("*** Sensor Type Parameters Read&Write example")

   #As Constant current does not need sensor, so Channel mode should be Heater or Tec
   result = TC300SetMode(hdl, 1, 0) #channel mode: 0: Heater; 1: Tec; 2: Constant current; 3: Synchronize with Ch1(only for channel 2)
   if(result<0):
       print("Set mode fail" , result)
   else:
       print("mode is" , "Heater")

   SensorT_str = input("Please set the sensor type here(just input the number): (0: PT100; 1: PT1000; 2: NTC1; 3: NTC2; 4: Thermo; 5: AD590; 6: EXT1; 7: EXT2)")
   SensorT = int(SensorT_str)

   result = TC300SetSensorType(hdl, 1, SensorT) # 0: PT100; 1: PT1000; 2: NTC1; 3: NTC2; 4: Thermo; 5: AD590; 6: EXT1; 7: EXT2
   if(result<0):
       print("Set Sensor Type fail" , result)
   else:
       print("Set Sensor Type successfully")

   SensorType=[0]
   SensorTypeList={0: "PT100", 1: "PT1000", 2: "NTC1", 3: "NTC2", 4: "Thermo", 5: "AD590", 6:"EXT1", 7:"EXT2"}
   result=TC300GetSensorType(hdl,1,SensorType)
   if(result<0):
      print("Get Sensor Type fail",result)
   else:
      print("Get Sensor Type :",SensorTypeList.get(SensorType[0]))

   #PT100/PT1000 parameter settings(sensor parameter(2 wire or 4 wire) and sensor offset)  
   if(SensorT == 0 or SensorT == 1): 
      result = TC300SetSensorParameter(hdl, 1, 0) #0: 2 wire; 1: 3 wire; 2: 4 wire; 3: J type; 4: K type
      if(result<0):
          print("Set Sensor Parameter fail" , result)
      else:
          print("Sensor Parameter is" , " 2 wire")

      SensorParameter=[0]
      SensorParameterList={0: "2 wire", 1: "3 wire", 2: "4 wire", 3: "J type", 4: "K type"}
      result=TC300GetSensorParameter(hdl,1,SensorParameter)
      if(result<0):
         print("Get Sensor Parameter fail",result)
      else:
         print("Sensor Parameter is :",SensorParameterList.get(SensorParameter[0]))

      result = TC300SetSensorOffset(hdl, 1, 5) #the offset value for PT100/PT1000, range: -10-10°C.
      if(result<0):
          print("Set Sensor offset fail" , result)
      else:
          print("Sensor offset is" , 5)

      SensorOffset=[0]
      result=TC300GetSensorOffset(hdl,1,SensorOffset)
      if(result<0):
         print("Get Sensor offset fail",result)
      else:
         print("Sensor offset is :",SensorOffset[0]) 

   #Thermo parameter settings(sensor parameter: J type or K type)
   elif(SensorT == 4):                       
      result = TC300SetSensorParameter(hdl, 1, 3) #0: 2 wire; 1: 3 wire; 2: 4 wire; 3: J type; 4: K type
      if(result<0):
         print("Set Sensor Parameter fail" , result)
      else:
         print("Sensor Parameter is" , " J type")

      SensorParameter=[0]
      SensorParameterList={0: "2 wire", 1: "3 wire", 2: "4 wire", 3: "J type", 4: "K type"}
      result=TC300GetSensorParameter(hdl,1,SensorParameter)
      if(result<0):
         print("Get Sensor Parameter fail",result)
      else:
         print("Sensor Parameter is :",SensorParameterList.get(SensorParameter[0]))

   #NTC1 parameter settings(sensor constant,T0 and R0)
   elif(SensorT == 2):                                               
      result = TC300SetNtcBeta(hdl, 1, 1000) #the β value for NTC1 sensor 0~9999.
      if(result<0):
          print("Set NTC Beta fail" , result)
      else:
          print("NTC Beta is" , 1000)

      NtcBeta=[0]
      result=TC300GetNtcBeta(hdl,1,NtcBeta)
      if(result<0):
         print("Get Sensor Constant fail",result)
      else:
         print("Sensor Constant is :",NtcBeta[0])  
  
      result = TC300SetT0Constant(hdl, 1, 100)  # the T0 value when sensor type is NTC1,0-999
      if(result<0):
          print("Set T0 Constant fail" , result)
      else:
          print("Sensor T0 Constant is" , 100)

      T0Constant=[0]
      result=TC300GetT0Constant(hdl,1,T0Constant)
      if(result<0):
         print("Get T0 Constant fail",result)
      else:
         print("Min T0 Constant is :",T0Constant[0])

      result = TC300SetR0Constant(hdl, 1, 200)  # the R0 value when sensor type is NTC1,0-999
      if(result<0):
          print("Set R0 Constant fail" , result)
      else:
          print("Sensor R0 Constant is" , 200)

      R0Constant=[0]
      result=TC300GetR0Constant(hdl,1,R0Constant)
      if(result<0):
         print("Get R0 Constant fail",result)
      else:
         print("Min R0 Constant is :",R0Constant[0])

   #EXT1 parameter settings(sensor constant,T0 and R0)
   elif(SensorT == 6):                                           
   
      result = TC300SetExtBeta(hdl, 1, 2000) #the β value for EXT1 sensor 0~9999.
      if(result<0):
          print("Set EXT1 Beta fail" , result)
      else:
          print("EXT1 Beta is" , 2000)

      EXT1Beta=[0]
      result=TC300GetExtBeta(hdl,1,EXT1Beta)
      if(result<0):
         print("Get Ext1 Beta fail",result)
      else:
         print("Ext1 Beta :",EXT1Beta[0])  

  
      result = TC300SetT0Constant(hdl, 1, 100)  # the T0 value when sensor type is EXT1,0-999
      if(result<0):
          print("Set T0 Constant fail" , result)
      else:
          print("Sensor T0 Constant is" , 100)

      T0Constant=[0]
      result=TC300GetT0Constant(hdl,1,T0Constant)
      if(result<0):
         print("Get T0 Constant fail",result)
      else:
         print("Min T0 Constant is :",T0Constant[0])

      result = TC300SetR0Constant(hdl, 1, 200)  # the R0 value when sensor type is EXT1,0-999
      if(result<0):
          print("Set R0 Constant fail" , result)
      else:
          print("Sensor R0 Constant is" , 200)

      R0Constant=[0]
      result=TC300GetR0Constant(hdl,1,R0Constant)
      if(result<0):
         print("Get R0 Constant fail",result)
      else:
         print("Min R0 Constant is :",R0Constant[0])

   #NTC2/EXT2 parameter settings(Hart A,Hart B and Hart C)
   elif(SensorT == 3 or SensorT == 7):                
      result = TC300SetHartAConstant(hdl, 1, -1.5555) #the Hart A value for NTC2 or EXT2 sensor -9.9999~9.9999
      if(result<0):
          print("Set Hart A value fail" , result)
      else:
          print("Hart A value is" , -1.5555)

      HartA=[0]
      result=TC300GetHartAConstant(hdl,1,HartA)
      if(result<0):
         print("Get Hart A value fail",result)
      else:
         print("Hart A value is :",HartA[0])  
  
      result = TC300SetHartBConstant(hdl, 1, -2.5555) #the Hart B value for NTC2 or EXT2 sensor -9.9999~9.9999
      if(result<0):
         print("Set Hart B value fail" , result)
      else:
         print("Hart B value is" , -2.5555)

      HartB=[0]
      result=TC300GetHartBConstant(hdl,1,HartB)
      if(result<0):
         print("Get Hart B value fail",result)
      else:
         print("Hart B value is :",HartB[0])

      result = TC300SetHartCConstant(hdl, 1, 3.5555) #the Hart C value for NTC2 or EXT2 sensor -9.9999~9.9999
      if(result<0):
          print("Set Hart C value fail" , result)
      else:
          print("Hart C value is" , 3.5555)

      HartC=[0]
      result=TC300GetHartCConstant(hdl,1,HartC)
      if(result<0):
         print("Get Hart C value fail",result)
      else:
         print("Hart C value is :",HartC[0])
    
   elif(SensorT == 5):
       print("No parameter need to be set when sensor type is AD590")
   else:
       print("Wrong sensor number")



      # ------------ Example Channel PID Read&Write demo for channel 1-------------- # 
def ChannelPIDReadWrite(hdl):
   print("*** Channel PID Read&Write example")  

   result = TC300SetPIDParameterD(hdl, 1, 6.66)  #  the PID parameter Td 0~9.99 (A*sec)/K
   if(result<0):
       print("Set Td fail" , result)
   else:
       print("Td is" ,6.66)

   Td=[0]
   result=TC300GetPIDParameterD(hdl,1,Td)
   if(result<0):
      print("Get Td fail",result)
   else:
      print("Td is :",Td[0])

   result = TC300SetPIDParameterI(hdl, 1, 7.77)  #  the PID parameter Ti 0~9.99 A/(K*sec)
   if(result<0):
       print("Set Ti fail" , result)
   else:
       print("Ti is" , 7.77)

   Ti=[0]
   result=TC300GetPIDParameterI(hdl,1,Ti)
   if(result<0):
      print("Get Ti fail",result)
   else:
      print("Ti is :",Ti[0])

   result = TC300SetPIDParameterP(hdl, 1, 8.88)  #  the PID parameter Kp 0~9.99 A/K
   if(result<0):
       print("Set Kp fail" , result)
   else:
       print("Kp is" , 8.88)

   Kp=[0]
   result=TC300GetPIDParameterP(hdl,1,Kp)
   if(result<0):
      print("Get Kp fail",result)
   else:
      print("Kp is :",Kp[0])

   result = TC300SetPIDParameterPeriod(hdl, 1, 4000)  #  the PID period time 100~5000 ms
   if(result<0):
       print("Set PID Parameter Period fail" , result)
   else:
       print("PID Parameter Period is" , 4000)

   Period=[0]
   result=TC300GetPIDParameterPeriod(hdl,1,Period)
   if(result<0):
      print("Get PID Parameter Period fail",result)
   else:
      print("PID Parameter Period is :",Period[0])   


def main():
    print("*** TC300 device python example ***")
    try:
        devs = TC300ListDevices()
        print(devs)
        if(len(devs)<=0):
           print('There is no devices connected')
           exit()     

        TC300= devs[0]
        serialNumber= TC300[0]
        hdl = TC300Open(serialNumber,115200,3)       
        if(hdl < 0):
          print("Connect ",serialNumber, "failed" )
          return -1;
        else:
          print("Connect ",serialNumber, "successfully")
   
        result = TC300IsOpen(serialNumber)
        if(result<0):
          print("Open failed ")
        else:
          print("TC300 Is Open ")
        
        ChannelReadWrite(hdl)
        print("--------------------------- Channel 1 Read&Write finished-------------------------")
        ChannelParametersReadWrite(hdl)
        print("--------------------------- Channel 1 Parameter Read&Write finished-------------------------")
        SensorTypeParametersReadWrite(hdl)
        print("--------------------------- channel 1 Sensor Type Parameters Read&Write finished-------------------------")        
        ChannelPIDReadWrite(hdl)
        print("--------------------------- Channel 1 PID Read&Write finished-------------------------")
        
        TC300Close(hdl)    
        
               
    except Exception as ex:
        print("Warning:", ex)
    print("*** End ***")
    input()
main()


